"""Test automatically write out of restart files"""
from __future__ import print_function

from ase import Atoms
from gpaw import GPAW
from gpaw.test import equal

restart_wf = 'gpaw-restart-wf.gpw'
# H2
H = Atoms('HH', [(0, 0, 0), (0, 0, 1)])
H.center(vacuum=2.0)

if 1:
    calc = GPAW(nbands=2,
                convergence={'eigenstates': 0.001,
                             'energy': 0.1,
                             'density': 0.1})
    H.set_calculator(calc)
    H.get_potential_energy()
    calc.write(restart_wf, 'all')

    # refine the result directly
    calc.set(convergence={'energy': 0.00001})
    Edirect = H.get_potential_energy()

# refine the result after reading from a file
H = GPAW(restart_wf, convergence={'energy': 0.00001}).get_atoms()
Erestart = H.get_potential_energy()

print(Edirect, Erestart)
# Note: the different density mixing introduces small differences
equal(Edirect, Erestart, 4e-5)



import platform
import time

from gpaw import __version__


class FMF:
    """Full-Metadata Format

    Full-Metadata Format after
    http://www.sir.uni-freiburg.de/repository/2009/SI20090302a/SI20090302a.pdf
    """
    def __init__(self,
                 title='-',
                 creator=None,
                 place=None,
                 escape='#'):
        self.escape = escape
        self.estimate_creator(creator)
        self.title = title
        self.place = place
 
    def header(self):
        ec = self.escape
        header = ec + '; -*- fmf version: 1.0 -*-\n'
        header += ec + '[*reference]\n'
        header += ec + 'creator: ' + self.creator + '\n'
        header += ec + 'created: ' + time.strftime('%Y-%m-%d %H:%M') + '\n'

        title = self.title
        if isinstance(title, str):
            title = title.split('\n')
        for i, line in enumerate(title):
            if i == 0:
                header += ec + 'title: '
            else:
                header += ec + '       '
            header += line + '\n'

        header += ec + 'gpaw-version: ' + __version__ + '\n'
        try:
            import socket
            header += ec + 'hostname: ' + socket.gethostname() + '\n'
        except:
            pass
        header += ec + 'architecture: ' + platform.uname()[4] + '\n'

        return header

    def data(self, definitions):
        ec = self.escape
        data = ec + '[* data definitions]\n'
        for definition in definitions:
            data += ec + definition + '\n'
        data += ec + '[* data]\n'
        return data

    def field(self, title, entries):
        ec = self.escape
        res = ec + '[' + title + ']\n'
        for entry in entries:
            res += ec + entry + '\n'
        return res

    def estimate_creator(self, creator=None):
        if creator is not None:
            self.creator = creator
            return

        try:
            # get
            import getpass
            username = getpass.getuser()
            
            try:
                import pwd
                gecos = pwd.getpwnam(username).pw_gecos
                hasalpha = False
                for letter in gecos:
                    if letter.isalpha():
                        hasalpha = True
                if hasalpha:
                    creator = gecos
                else:
                    creator = username
            except:
                creator = username

        except:
            creator = 'unknown'

        self.creator = creator



import numpy as np
from math import pi, cos, sin
from ase import Atom, Atoms
from gpaw import GPAW, setup_paths
from gpaw.poisson import FDPoissonSolver
from gpaw.xas import XAS
from gpaw.test import equal
from gpaw.atom.generator2 import generate
import gpaw.mpi as mpi

# Generate setup for oxygen with half a core-hole:
gen = generate('O', '2s,s,2p,p,d', [1.2], 1.0, None, 2, core_hole='1s,0.5')
setup = gen.make_paw_setup('hch1s')
setup.write_xml()
if setup_paths[0] != '.':
    setup_paths.insert(0, '.')

a = 5.0
d = 0.9575
t = pi / 180 * 104.51
H2O = Atoms([Atom('O', (0, 0, 0)),
             Atom('H', (d, 0, 0)),
             Atom('H', (d * cos(t), d * sin(t), 0))],
            cell=(a, a, a), pbc=False)
H2O.center()
calc = GPAW(nbands=10, h=0.2, setups={'O': 'hch1s'},
            experimental={'niter_fixdensity': 2},
            poissonsolver=FDPoissonSolver(use_charge_center=True))
H2O.set_calculator(calc)
e = H2O.get_potential_energy()
niter = calc.get_number_of_iterations()

if mpi.size == 1:
    xas = XAS(calc)
    x, y = xas.get_spectra()
    e1_n = xas.eps_n
    de1 = e1_n[1] - e1_n[0]

calc.write('h2o-xas.gpw')

if mpi.size == 1:
    calc = GPAW('h2o-xas.gpw', txt=None,
                poissonsolver=FDPoissonSolver(use_charge_center=True))
    calc.initialize()
    xas = XAS(calc)
    x, y = xas.get_spectra()
    e2_n = xas.eps_n
    w_n = np.sum(xas.sigma_cn.real**2, axis=0)
    de2 = e2_n[1] - e2_n[0]

    equal(de2, 2.064, 0.005)
    equal(w_n[1] / w_n[0], 2.22, 0.01)

    assert de1 == de2

if 0:
    import matplotlib.pyplot as plt
    plt.plot(x, y[0])
    plt.plot(x, sum(y))
    plt.show()



from __future__ import print_function
import numpy as np
from ase.build import molecule

from gpaw import GPAW
from gpaw.poisson import FDPoissonSolver
from gpaw.lcao.projected_wannier import get_lcao_projections_HSP

atoms = molecule('C2H2')
atoms.center(vacuum=3.0)
calc = GPAW(gpts=(32, 32, 48),
            experimental={'niter_fixdensity': 2},
            poissonsolver=FDPoissonSolver(),
            eigensolver='rmm-diis')
atoms.set_calculator(calc)
atoms.get_potential_energy()

V_qnM, H_qMM, S_qMM, P_aqMi = get_lcao_projections_HSP(
    calc, bfs=None, spin=0, projectionsonly=False)


# Test H and S
eig = sorted(np.linalg.eigvals(np.linalg.solve(S_qMM[0], H_qMM[0])).real)
eig_ref = np.array([-17.879394403125634, -13.248793622886552,
                    -11.431247684217933, -7.125771721594344,
                    -7.125771721594252, 0.5927469818425286,
                    0.5927469818425768, 3.925124575829719,
                    7.451028687071511, 26.734351930654853])
print(eig)
assert np.allclose(eig, eig_ref)



import numpy as np

from ase.build import molecule
from gpaw import GPAW
from gpaw.tddft import TDDFT as GRIDTDDFT
from gpaw.lcaotddft import LCAOTDDFT
from gpaw.poisson import PoissonSolver
from gpaw.poisson_extravacuum import ExtraVacuumPoissonSolver

name = 'Na2'
poissoneps = 1e-16
gpts = np.array([16, 16, 24])
# Uncomment the following line if you want to run the test with 4 cpus
# gpts *= 2


def PS(**kwargs):
    return PoissonSolver(eps=poissoneps, **kwargs)

poissonsolver_i = []

ps = PS()
poissonsolver_i.append(ps)

ps = PS(remove_moment=4)
poissonsolver_i.append(ps)

ps = ExtraVacuumPoissonSolver(gpts * 2, PS())
poissonsolver_i.append(ps)

ps = ExtraVacuumPoissonSolver(gpts * 2, PS(), PS(), 2)
poissonsolver_i.append(ps)

ps1 = ExtraVacuumPoissonSolver(gpts, PS(), PS(), 1)
ps = ExtraVacuumPoissonSolver(gpts, ps1, PS(), 1)
poissonsolver_i.append(ps)

for poissonsolver in poissonsolver_i:
    for mode in ['fd', 'lcao']:
        atoms = molecule(name)
        atoms.center(vacuum=3.0)

        # Standard ground state calculation
        # Use loose convergence criterion for speed
        calc = GPAW(nbands=2, gpts=gpts / 2, setups={'Na': '1'}, txt=None,
                    poissonsolver=poissonsolver,
                    mode=mode,
                    convergence={'energy': 1.0,
                                 'density': 1.0,
                                 'eigenstates': 1.0})
        atoms.set_calculator(calc)
        energy = atoms.get_potential_energy()
        descr = calc.hamiltonian.poisson.get_description()
        calc.write('%s_gs.gpw' % name, mode='all')

        # Restart ground state
        calc = GPAW('%s_gs.gpw' % name, txt=None)
        ps = calc.hamiltonian.poisson
        assert descr == ps.get_description(), \
            'poisson solver has changed in GPAW / %s' % mode

        # Time-propagation TDDFT
        if mode == 'lcao':
            TDDFT = LCAOTDDFT
        else:
            TDDFT = GRIDTDDFT
        calc = TDDFT('%s_gs.gpw' % name, txt=None)
        ps = calc.hamiltonian.poisson
        assert descr == ps.get_description(), \
            'poisson solver has changed in TDDFT / %s' % mode



from __future__ import print_function
import numpy as np
from ase import Atoms
from gpaw import GPAW
from gpaw.test import equal

bulk = Atoms('Li', pbc=True)
k = 4
g = 8
calc = GPAW(gpts=(g, g, g), kpts=(k, k, k),
                  mode='lcao', basis='dzp')
bulk.set_calculator(calc)
e = []
niter = []
A = [2.6, 2.65, 2.7, 2.75, 2.8]
for a in A:
    bulk.set_cell((a, a, a))
    e.append(bulk.get_potential_energy())
    niter.append(calc.get_number_of_iterations())

a = np.roots(np.polyder(np.polyfit(A, e, 2), 1))[0]
print('a =', a)
equal(a, 2.6378, 0.0003)

e_ref = [-1.8677343236247692, -1.8690343169380492, -1.8654175796625045,
         -1.8566274574918875, -1.8432374955346396]
niter_ref = [6, 6, 6, 6, 6]

print(e)
energy_tolerance = 0.0003
niter_tolerance = 0

for i in range(len(A)):
    equal(e[i], e_ref[i], energy_tolerance)
    equal(niter[i], niter_ref[i], niter_tolerance)

wf1 = calc.get_pseudo_wave_function(kpt=3, band=0)
calc.write('Li', mode='all')
calc2 = GPAW('Li')
calc2.initialize_positions()
wf2 = calc2.get_pseudo_wave_function(kpt=3, band=0)
equal(abs(wf1 - wf2).max(), 0, 1e-9)




from math import sqrt
from ase import Atoms
from ase.dft.bandgap import bandgap
from gpaw import GPAW, FermiDirac
from gpaw.test import equal

# Setup up bulk NiO in an antiferromagnetic configuration.
# Lattice constants:
a = 4.19
b = a / sqrt(2)
m = 2
k = 2  # number of k-points

atoms = Atoms(
    symbols='Ni2O2',
    pbc=True,
    cell=(b, b, a),
    positions=[(0, 0, 0),
               (b / 2, b / 2, a / 2),
               (0, 0, a / 2),
               (b / 2, b / 2, 0)],
    magmoms=(m, -m, 0, 0))

name = 'ni2o2'
for setup in ['10', '10:d,6.0']:
    calc = GPAW(
        mode='pw',
        occupations=FermiDirac(width=0.05),
        setups={'Ni': setup},
        convergence={'eigenstates': 8e-4,
                     'density': 1.0e-2,
                     'energy': 0.1},
        txt=name + '.txt',
        kpts=(k, k, k),
        xc='oldPBE')
    atoms.calc = calc
    e = atoms.get_potential_energy()
    gap, _, _ = bandgap(calc)
    print(name, gap)
    if name == 'ni2o2':
        equal(gap, 0.8, 0.1)
    else:
        equal(gap, 4.7, 0.2)
    name += '+U'



import numpy as np

from ase.lattice.hexagonal import Graphene
from ase.parallel import parprint as pp

from gpaw import GPAW
from gpaw.response.df import DielectricFunction
from gpaw.mpi import world

system = Graphene(symbol='C',
                  latticeconstant={'a': 2.45,'c': 1.0},
                  size=(1,1,1))
system.pbc = (1, 1, 0)
system.center(axis=2, vacuum=4.0)

nkpts = 5


communicator = world.new_communicator(np.array([world.rank]))
gpwname = 'dump.graphene.gpw'

if world.rank == 0:
    calc = GPAW(mode='pw',
                kpts=(nkpts, nkpts, 1),
                communicator=communicator,
                xc='oldLDA',
                nbands=len(system) * 6,
                txt='gpaw.graphene.txt')
    system.set_calculator(calc)
    system.get_potential_energy()
    calc.write(gpwname, mode='all')

world.barrier()

parallel = dict(domain=(1, 1, 1), band=1)
if world.size == 8:
    #parallel['domain'] = (1, 1, 2)
    parallel['band'] = 2
calc = GPAW(gpwname,
            txt=None,
            parallel=parallel,
            idiotproof=False)
pp('after restart')

q = np.array([1.0 / nkpts, 0., 0.])
w = np.linspace(0, 31.9, 320)
dw = w[1] - w[0]


def getpeak(energies, loss):
    arg = loss.argmax()
    energy = energies[arg]
    peakloss = loss[arg]
    return energy, peakloss

scriptlines = []

loss_errs = []
energy_errs = []


def check(name, energy, peakloss, ref_energy, ref_loss):
    pp('check %s :: energy = %5.2f [%5.2f], peakloss = %.12f [%.12f]'
       % (name, energy, ref_energy, peakloss, ref_loss))
    energy_errs.append(abs(energy - ref_energy))
    loss_errs.append(abs(peakloss - ref_loss))


array = np.array
template = """\
check_df('%s', %s, %s, %s, %s,
         **%s)"""


def check_df(name, ref_energy, ref_loss, ref_energy_lfe, ref_loss_lfe,
             **kwargs_override):
    kwargs = dict(calc=calc, frequencies=w.copy(), eta=0.5, ecut=30,
                  txt='df.%s.txt' % name)
    kwargs.update(kwargs_override)
    df = DielectricFunction(**kwargs)
    fname = 'dfdump.%s.dat' % name
    df.get_eels_spectrum('RPA',q_c=q,filename=fname)
    world.barrier()
    d = np.loadtxt(fname,delimiter=',')

    loss = d[:, 1]
    loss_lfe = d[:, 2]
    energies = d[:, 0]

    #import pylab as pl
    #fig = pl.figure()
    #ax1 = fig.add_subplot(111)
    #ax1.plot(d[:, 0], d[:, 1]/np.max(d[:, 1]))
    #ax1.plot(d[:, 0], d[:, 2]/np.max(d[:, 2]))
    #ax1.axis(ymin=0, ymax=1)
    #fig.savefig('fig.%s.pdf' % name)

    energy, peakloss = getpeak(energies, loss)
    energy_lfe , peakloss_lfe = getpeak(energies, loss_lfe)

    check(name, energy, peakloss, ref_energy, ref_loss)
    check('%s-lfe' % name, energy_lfe, peakloss_lfe, ref_energy_lfe,
          ref_loss_lfe)

    line = template % (name, energy, peakloss, energy_lfe, peakloss_lfe,
                       repr(kwargs_override))
    scriptlines.append(line)


# These lines can be generated by the loop over 'scriptlines' below,
# in case new reference values are wanted

# The implementation for vcut choices has still to be done

check_df('3d', 20.20, 2.505295593820, 26.90, 1.748517033160)
#        **{'rpad': array([1, 1, 1])}) #, 'vcut': '3D'})
#check_df('2d', 20.10, 2.449662058530, 26.80, 1.538080502420,
#         **{'rpad': array([1, 1, 1]), 'vcut': '2D'})


pp()
pp('Insert lines into script to set new reference values:')
for line in scriptlines:
    pp(line)
pp()

for err in energy_errs:
    # with the current grid this error just means
    assert err < dw / 4.0, err
for err in loss_errs:
    assert err < 1e-6, err



from ase.build import molecule
from gpaw import GPAW

import warnings
# Silence those warnings.
warnings.filterwarnings('ignore', 'Setup for',)

m = molecule('H')
m.center(vacuum=2.0)
calc = GPAW(mode='lcao')
m.set_calculator(calc)
m.get_potential_energy()
calc.write('r.gpw')
calc = GPAW('r.gpw', xc='PBE', idiotproof=False)
calc.get_potential_energy()



"""Check for tunability of gamma for yukawa potential."""
from ase import Atoms
from gpaw import GPAW, setup_paths
from gpaw.cluster import Cluster
from gpaw.eigensolvers import RMMDIIS
from gpaw.xc.hybrid import HybridXC
from gpaw.occupations import FermiDirac
from gpaw.test import equal, gen
import _gpaw

libxc_version = getattr(_gpaw, 'libxc_version', '2.x.y')
if int(libxc_version.split('.')[0]) < 3:
    from unittest import SkipTest
    raise SkipTest

if setup_paths[0] != '.':
    setup_paths.insert(0, '.')

for atom in ['Be']:
    gen(atom, xcname='PBE', scalarrel=True, exx=True,
        yukawa_gamma=0.83, gpernode=149)

h = 0.35
be = Cluster(Atoms('Be', positions=[[0, 0, 0]]))
be.minimal_box(3.0, h=h)

c = {'energy': 0.05, 'eigenstates': 0.05, 'density': 0.05}

IP = 8.79

xc = HybridXC('LCY-PBE', omega=0.83)
fname = 'Be_rsf.gpw'

calc = GPAW(txt='Be.txt', xc=xc, convergence=c,
            eigensolver=RMMDIIS(), h=h,
            occupations=FermiDirac(width=0.0), spinpol=False)
be.set_calculator(calc)
# energy = na2.get_potential_energy()
# calc.set(xc=xc)
energy_083 = be.get_potential_energy()
(eps_homo, eps_lumo) = calc.get_homo_lumo()
equal(eps_homo, -IP, 0.15)
xc2 = 'LCY-PBE'
energy_075 = calc.get_xc_difference(HybridXC(xc2))
equal(energy_083 - energy_075, 21.13, 0.2, 'wrong energy difference')
calc.write(fname)
calc2 = GPAW(fname)
func = calc2.get_xc_functional()
assert func['name'] == 'LCY-PBE', 'wrong name for functional'
assert func['omega'] == 0.83, 'wrong value for RSF omega'



from __future__ import print_function
from ase import Atoms
from gpaw import GPAW, FermiDirac

a = 2.0
L = 5.0

a1 = Atoms('H', pbc=(1, 0, 0), cell=(a, L, L))
a1.center()
atoms = a1.repeat((4, 1, 1))


def energy(symm={}):
    atoms.set_calculator(GPAW(h=0.3,
                              occupations=FermiDirac(width=0.1),
                              symmetry=symm,
                              convergence=dict(energy=1e-6),
                              kpts=(3, 1, 1),
                              mode='lcao'))
    return atoms.get_potential_energy()

e1 = energy()
e2 = energy('off')

print(e1)
print(e2)

assert abs(e2 - e1) < 1e-6



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Run with even number of cores

from gpaw import GPAW, Mixer, PoissonSolver
import ase.parallel as mpi
from ase.build import fcc211, add_adsorbate
from ase.constraints import FixAtoms
from ase.optimize import QuasiNewton
from ase.neb import NEBtools
from ase.autoneb import AutoNEB

size = mpi.world.size
rank = mpi.world.rank

slab = fcc211('Ag', size=(3, 2, 1), vacuum=3.0)
add_adsorbate(slab, 'Ag', 0.5, (-0.1, 2.7))

slab.set_constraint(FixAtoms(range(6)))
slab.pbc = 1

def getcalc(**kwargs):
    kwargs1 = dict(xc='oldLDA',
                   gpts=(32, 24, 32),
                   setups={'Ag': '11'},
                   #h=0.24,
                   poissonsolver=PoissonSolver(relax='GS'),
                   mixer=Mixer(0.5, 5, 50.0),
                   mode='lcao',
                   basis='sz(dzp)')
    kwargs1.update(kwargs)
    return GPAW(**kwargs1)

calc = getcalc()

slab.set_calculator(calc)

qn = QuasiNewton(slab, trajectory='neb000.traj')
qn.run(fmax=0.05)

slab[-1].x += slab.get_cell()[0, 0]
slab[-1].y += 2.8

qn = QuasiNewton(slab, trajectory='neb001.traj')
qn.run(fmax=0.05)


def attach_calculators(images):
    nim = len(images)
    n = size // nim  # number of cpu's per image
    j = rank // n  # image number
    assert nim * n == size

    for i in range(nim):
        ranks = range(i * n, (i + 1) * n)
        if rank in ranks:
            calc = getcalc(txt='neb%d.txt' % j,
                           communicator=ranks)
            images[i].set_calculator(calc)

autoneb=AutoNEB(attach_calculators,
                prefix='neb',
                n_simul=2,
                parallel=True,
                climb=True,
                n_max=5,
                optimizer='FIRE',
                fmax=0.05,
                k=0.5,
                maxsteps=[25, 1000])
autoneb.run()

nebtools = NEBtools(autoneb.all_images)
barrier, delta_e = nebtools.get_barrier()
print('barrier', barrier)
ref = 0.74051020956857272  # 1.484 <- with better parameters
err = abs(barrier - ref)
assert err < 1e-3, 'barrier={}, expected={}, err={}'.format(barrier, ref, err)



from gpaw.xc.functional import XCFunctional
from gpaw.mpi import world
import numpy as np


class NonLocalFunctional(XCFunctional):
    def __init__(self, xcname):
        self.contributions = []
        self.xcs = {}
        XCFunctional.__init__(self, xcname, 'GLLB')
        self.mix = None
        self.mix_vt_sg = None
        self.old_vt_sg = None
        self.old_H_asp = {}

    def set_mix(self, mix):
        self.mix = mix

    def initialize(self, density, hamiltonian, wfs, occupations):
        self.gd = density.gd  # smooth grid describtor
        self.finegd = density.finegd  # fine grid describtor
        self.nt_sg = density.nt_sg  # smooth density
        self.setups = wfs.setups  # All the setups
        self.nspins = wfs.nspins  # number of spins
        self.wfs = wfs
        self.occupations = occupations
        self.density = density
        self.hamiltonian = hamiltonian
        self.nvalence = wfs.nvalence

        # self.vt_sg = paw.vt_sg # smooth potential
        # self.kpt_u = kpt_u # kpoints object
        # self.interpolate = interpolate # interpolation function
        # self.nuclei = nuclei

        # Is this OK place?
        self.initialize0()

    def set_positions(self, spos_ac):
        for contribution in self.contributions:
            contribution.set_positions(spos_ac)

    def pass_stuff_1d(self, ae):
        self.ae = ae

    def initialize0(self):
        for contribution in self.contributions:
            contribution.initialize()

    def initialize_1d(self):
        for contribution in self.contributions:
            contribution.initialize_1d()

    def calculate_impl(self, gd, n_sg, v_sg, e_g):
        if self.nspins == 1:
            self.calculate_spinpaired(e_g, n_sg[0], v_sg[0])
        else:
            self.calculate_spinpolarized(e_g, n_sg, v_sg)
        return gd.integrate(e_g)

    def calculate_paw_correction(self, setup, D_sp, dEdD_sp, a=None,
                                 addcoredensity=True):
        return self.calculate_energy_and_derivatives(setup, D_sp, dEdD_sp, a,
                                                     addcoredensity)

    def calculate_spinpaired(self, e_g, n_g, v_g):
        e_g[:] = 0.0
        if self.mix is None:
            for contribution in self.contributions:
                contribution.calculate_spinpaired(e_g, n_g, v_g)
        else:
            cmix = self.mix
            if self.mix_vt_sg is None:
                self.mix_vt_sg = np.zeros_like(v_g)
                self.old_vt_sg = np.zeros_like(v_g)
                cmix = 1.0
            self.mix_vt_sg[:] = 0.0
            for contribution in self.contributions:
                contribution.calculate_spinpaired(e_g, n_g, self.mix_vt_sg)
            self.mix_vt_sg = (cmix * self.mix_vt_sg +
                              (1.0 - cmix) * self.old_vt_sg)
            v_g += self.mix_vt_sg
            self.old_vt_sg[:] = self.mix_vt_sg

    def calculate_spinpolarized(self, e_g, n_sg, v_sg):
        e_g[:] = 0.0
        for contribution in self.contributions:
            contribution.calculate_spinpolarized(e_g, n_sg, v_sg)

    def calculate_energy_and_derivatives(self, setup, D_sp, H_sp, a,
                                         addcoredensity=True):
        if setup.xc_correction is None:
            return 0.0
        Exc = 0.0
        # We are supposed to add to H_sp, not write directly
        H0_sp = H_sp
        H_sp = H0_sp.copy()
        H_sp[:] = 0.0

        if self.mix is None:
            for contribution in self.contributions:
                Exc += contribution.calculate_energy_and_derivatives(
                    setup, D_sp, H_sp, a, addcoredensity)
        else:
            cmix = self.mix
            if a not in self.old_H_asp:
                self.old_H_asp[a] = H_sp.copy()
                cmix = 1.0

            for contribution in self.contributions:
                Exc += contribution.calculate_energy_and_derivatives(
                    setup, D_sp, H_sp, a, addcoredensity)
            H_sp *= cmix
            H_sp += (1 - cmix) * self.old_H_asp[a]
            self.old_H_asp[a][:] = H_sp.copy()

        # if a == 0:
        #     print('eh', H_sp.sum())
        H0_sp += H_sp
        Exc -= setup.xc_correction.e_xc0
        return Exc

    def get_xc_potential_and_energy_1d(self, v_g):
        Exc = 0.0
        for contribution in self.contributions:
            Exc += contribution.add_xc_potential_and_energy_1d(v_g)
        return Exc

    def get_smooth_xc_potential_and_energy_1d(self, vt_g):
        Exc = 0.0
        for contribution in self.contributions:
            Exc += contribution.add_smooth_xc_potential_and_energy_1d(vt_g)
        return Exc

    def initialize_from_atomic_orbitals(self, basis_functions):
        for contribution in self.contributions:
            contribution.initialize_from_atomic_orbitals(basis_functions)

    def get_extra_setup_data(self, dict):
        for contribution in self.contributions:
            contribution.add_extra_setup_data(dict)

    def add_contribution(self, contribution):
        self.contributions.append(contribution)
        self.xcs[contribution.get_name()] = contribution

    def print_functional(self):
        if world.rank != 0:
            return
        print()
        print("Functional being used consists of")
        print("---------------------------------------------------")
        print("| Weight    | Module           | Description      |")
        print("---------------------------------------------------")
        for contribution in self.contributions:
            print("|%9.3f  | %-17s| %-17s|" %
                  (contribution.weight, contribution.get_name(),
                   contribution.get_desc()))
        print("---------------------------------------------------")
        print()

    def read(self, reader):
        for contribution in self.contributions:
            contribution.read(reader)

    def write(self, writer):
        for contribution in self.contributions:
            contribution.write(writer)

    def heeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeelp(self, olddens):
        for contribution in self.contributions:
            try:
                contribution.heeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeelp(olddens)
            except AttributeError:
                pass



# http://www.answers.com/topic/hartree-units

from math import pi

from ase.units import _hbar, _eps0, _me, _e, Hartree

_autime = _hbar**3*(4*pi*_eps0)**2/(_me*_e**4) # 1 autime ~ 2.42e-17 s

# Conversion factors between ASE and GPAW units

attosec_to_autime = 1e-18/_autime # 1 as ~ 0.0413 autime
autime_to_attosec = _autime*1e18 # 1 autime ~ 24.2 as

eV_to_hartree = 1.0/Hartree # 1 eV ~ 0.0368 Eh
hartree_to_eV = Hartree # 1 Eh ~ 27.2 eV

hartree_to_aufrequency = 1.0 # Eh ~ autime^(-1) since hbar=1
aufrequency_to_hartree = 1.0 # autime^(-1) ~ Eh since hbar=1

eV_to_aufrequency = 1.0/Hartree # 1 eV ~ 0.0368 autime^(-1)
aufrequency_to_eV = Hartree # 1 autime^(-1) ~ 27.2 eV

assert eV_to_aufrequency == eV_to_hartree * hartree_to_aufrequency
assert aufrequency_to_eV == aufrequency_to_hartree * hartree_to_eV

# Short-hand names
eV_to_au = eV_to_aufrequency
au_to_eV = aufrequency_to_eV
as_to_au = attosec_to_autime
au_to_as = autime_to_attosec
au_to_fs = au_to_as / 1e3



from __future__ import print_function
import numpy as np

# Set-up a simple matrix in parallel, diagonalize using ScaLAPACK
# D&C driver then compare *eigenvalues* with serial LAPACK diagonlize
from time import time

from gpaw.blacs import BlacsGrid
from gpaw.mpi import world
from gpaw.utilities import compiled_with_sl
from gpaw.utilities.lapack import diagonalize
from gpaw.utilities.scalapack import scalapack_set, \
    scalapack_zero

switch_uplo = {'U': 'L', 'L': 'U'}

rank = world.rank

# tol
tol = 5e-12 # eigenvalue tolerance

def main(nbands=1000, mprocs=2, mb=64):
    # Set-up BlacsGrud
    grid = BlacsGrid(world, mprocs, mprocs)

    # Create descriptor
    nndesc = grid.new_descriptor(nbands, nbands, mb, mb)
    H_nn = nndesc.empty(dtype=float) # outside the BlacsGrid these are size zero
    C_nn = nndesc.empty(dtype=float) # outside the BlacsGrid these are size zero
    eps_N  = np.empty((nbands), dtype=float) # replicated array on all MPI tasks
    # Fill ScaLAPACK array
    alpha = 0.1 # off-diagonal
    beta = 75.0 # diagonal
    uplo = 'L' # lower-triangular
    scalapack_set(nndesc, H_nn, alpha, beta, uplo)
    scalapack_zero(nndesc, H_nn, switch_uplo[uplo])

    t1 = time()
    # either interface will work, we recommend use the latter interface
    # scalapack_diagonalize_dc(nndesc, H_nn.copy(), C_nn, eps_N, 'L')
    nndesc.diagonalize_dc(H_nn.copy(), C_nn, eps_N)
    t2 = time()
    world.broadcast(eps_N, 0) # all MPI tasks now have eps_N
    world.barrier() # wait for everyone to finish

    if rank == 0:
        print('ScaLAPACK diagonalize_dc', t2-t1)

    # Create replicated NumPy array
    diagonal = np.eye(nbands,dtype=float)
    offdiagonal = np.tril(np.ones((nbands,nbands)), -1)
    H0 = beta*diagonal + alpha*offdiagonal
    E0 = np.empty((nbands), dtype=float)

    t1 = time()
    diagonalize(H0,E0)
    t2 = time()

    if rank == 0:
        print('LAPACK diagonalize', t2-t1)

    delta = abs(E0-eps_N).max()
    if rank == 0:
        print(delta)
        assert delta < tol

if __name__ in ['__main__', '__builtin__']:
    if not compiled_with_sl():
        print('Not built with ScaLAPACK. Test does not apply.')
    else:
        main()



"""This module defines different external potentials."""
import warnings

import numpy as np

from ase.units import Bohr, Hartree

import _gpaw

__all__ = ['ConstantPotential', 'ConstantElectricField', 'CDFTPotential']


def create_external_potential(name, **kwargs):
    """Construct potential from dict."""
    if name not in __all__:
        raise ValueError
    if name == 'CDFTPotential':
        return None
    return globals()[name](**kwargs)


class ExternalPotential:
    vext_g = None
    vext_q = None

    def get_potential(self, gd):
        """Get the potential on a regular 3-d grid.

        Will only call calculate_potential() the first time."""

        if self.vext_g is None:
            self.calculate_potential(gd)
            self.vext_g.flags.writeable = False
        return self.vext_g

    def get_potentialq(self, gd, pd3):
        """Get the potential on a regular 3-d grid in real space.

        Will only call calculate_potential() the first time."""

        if self.vext_q is None:
            vext_g = self.get_potential(gd)
            self.vext_q = pd3.fft(vext_g)
            self.vext_q.flags.writeable = False

        return self.vext_q

    def calculate_potential(self, gd):
        raise NotImplementedError

    def get_name(self):
        return self.__class__.__name__


class ConstantPotential(ExternalPotential):
    """Constant potential for tests."""
    def __init__(self, constant=1.0):
        self.constant = constant / Hartree
        self.name = 'ConstantPotential'

    def __str__(self):
        return 'Constant potential: {:.3f} V'.format(self.constant * Hartree)

    def calculate_potential(self, gd):
        self.vext_g = gd.zeros() + self.constant

    def todict(self):
        return {'name': self.name,
                'constant': self.constant * Hartree}


class ConstantElectricField(ExternalPotential):
    def __init__(self, strength, direction=[0, 0, 1], tolerance=1e-7):
        """External constant electric field.

        strength: float
            Field strength in V/Ang.
        direction: vector
            Polarisation direction.
        """
        d_v = np.asarray(direction)
        self.field_v = strength * d_v / (d_v**2).sum()**0.5 * Bohr / Hartree
        self.tolerance = tolerance
        self.name = 'ConstantElectricField'

    def __str__(self):
        return ('Constant electric field: '
                '({:.3f}, {:.3f}, {:.3f}) V/Ang'
                .format(*(self.field_v * Hartree / Bohr)))

    def calculate_potential(self, gd):
        # Currently skipped, PW mode is periodic in all directions
        # d_v = self.field_v / (self.field_v**2).sum()**0.5
        # for axis_v in gd.cell_cv[gd.pbc_c]:
        #     if abs(np.dot(d_v, axis_v)) > self.tolerance:
        #         raise ValueError(
        #             'Field not perpendicular to periodic axis: {}'
        #             .format(axis_v))

        center_v = 0.5 * gd.cell_cv.sum(0)
        r_gv = gd.get_grid_point_coordinates().transpose((1, 2, 3, 0))
        self.vext_g = np.dot(r_gv - center_v, self.field_v)

    def todict(self):
        strength = (self.field_v**2).sum()**0.5
        return {'name': self.name,
                'strength': strength * Hartree / Bohr,
                'direction': self.field_v / strength}


class PointChargePotential(ExternalPotential):
    def __init__(self, charges, positions=None,
                 rc=0.2, rc2=np.inf, width=1.0):
        """Point-charge potential.

        charges: list of float
            Charges.
        positions: (N, 3)-shaped array-like of float
            Positions of charges in Angstrom.  Can be set later.
        rc: float
            Inner cutoff for Coulomb potential in Angstrom.
        rc2: float
            Outer cutoff for Coulomb potential in Angstrom.
        width: float
            Width for cutoff function for Coulomb part.

        For r < rc, 1 / r is replaced by a third order polynomial in r^2 that
        has matching value, first derivative, second derivative and integral.

        For rc2 - width < r < rc2, 1 / r is multiplied by a smooth cutoff
        function (a third order polynomium in r).

        You can also give rc a negative value.  In that case, this formula
        is used::

            (r^4 - rc^4) / (r^5 - |rc|^5)

        for all values of r - no cutoff at rc2!
        """
        self.q_p = np.ascontiguousarray(charges, float)
        self.rc = rc / Bohr
        self.rc2 = rc2 / Bohr
        self.width = width / Bohr
        if positions is not None:
            self.set_positions(positions)
        else:
            self.R_pv = None

        if abs(self.q_p).max() < 1e-14:
            warnings.warn('No charges!')
        if self.rc < 0. and self.rc2 < np.inf:
            warnings.warn('Long range cutoff chosen but will not be applied\
                           for negative inner cutoff values!')

    def __str__(self):
        return ('Point-charge potential '
                '(points: {}, cutoffs: {:.3f}, {:.3f}, {:.3f} Ang)'
                .format(len(self.q_p),
                        self.rc * Bohr,
                        (self.rc2 - self.width) * Bohr,
                        self.rc2 * Bohr))

    def set_positions(self, R_pv, com_pv=None):
        """Update positions."""
        if com_pv is not None:
            self.com_pv = np.asarray(com_pv) / Bohr
        else:
            self.com_pv = None

        self.R_pv = np.asarray(R_pv) / Bohr
        self.vext_g = None

    def _molecule_distances(self, gd):
        if self.com_pv is not None:
            return self.com_pv - gd.cell_cv.sum(0) / 2

    def calculate_potential(self, gd):
        assert gd.orthogonal
        self.vext_g = gd.zeros()

        dcom_pv = self._molecule_distances(gd)

        _gpaw.pc_potential(gd.beg_c, gd.h_cv.diagonal().copy(),
                           self.q_p, self.R_pv,
                           self.rc, self.rc2, self.width,
                           self.vext_g, dcom_pv)

    def get_forces(self, calc):
        """Calculate forces from QM charge density on point-charges."""
        dens = calc.density
        F_pv = np.zeros_like(self.R_pv)
        gd = dens.finegd
        dcom_pv = self._molecule_distances(gd)

        _gpaw.pc_potential(gd.beg_c, gd.h_cv.diagonal().copy(),
                           self.q_p, self.R_pv,
                           self.rc, self.rc2, self.width,
                           self.vext_g, dcom_pv, dens.rhot_g, F_pv)
        gd.comm.sum(F_pv)
        return F_pv * Hartree / Bohr

class CDFTPotential(ExternalPotential):
    # Dummy class to make cDFT compatible with new external potential class ClassName(object):
    def __init__(self):
        self.name = 'CDFTPotential'



from __future__ import print_function
# Test that LCAO wavefunctions are available and equal after restarts
# in normal as well as 'all' mode

import numpy as np
from ase.build import molecule
from gpaw import GPAW

# setting number of decimals globally makes numpy.test() tests
# which use docstrings fail
# np.set_printoptions(precision=3, suppress=1)

system = molecule('H2')
system.center(vacuum=2.5)

calc = GPAW(mode='lcao', basis='sz(dzp)', h=0.3, nbands=1, txt=None)
system.set_calculator(calc)
system.get_potential_energy()
wf = calc.get_pseudo_wave_function(0)

for mode in ['all', 'normal']:
    fname = 'lcao-restart.%s.gpw' % mode
    calc.write(fname, mode=dict(normal='', all='all')[mode])
    
    calc2 = GPAW(fname, txt=None)
    if mode == 'normal':
        continue
    wf2 = calc2.get_pseudo_wave_function(0)
    err = np.abs(wf2 - wf).max()
    print('%s: err=%s' % (mode, repr(err)))
    assert abs(err) < 1e-14



from __future__ import print_function

from ase import Atom, Atoms
import numpy as np

from gpaw import GPAW, FermiDirac, PoissonSolver
from gpaw.utilities.dos import raw_orbital_LDOS, raw_wignerseitz_LDOS, RawLDOS
from gpaw.test import equal
import gpaw.mpi as mpi

comms = [mpi.world.new_communicator(np.array([r])) for r in range(mpi.size)]
comm = comms[mpi.rank]

Hnospin = Atoms([Atom('H')], cell=[5, 5, 5], pbc=False)
Hspin = Atoms([Atom('H', magmom=1)], cell=[5, 5, 5], pbc=False)
LiH = Atoms([Atom('Li', [.0, .0, .41]),
             Atom('H', [.0, .0, -1.23])])
Hnospin.center()
Hspin.center()
LiH.center(vacuum=3.0)

# This is needed for the Wigner-Seitz test to give
# architecture-independent results:
LiH.translate(0.003234)

calc = GPAW(gpts=(24, 24, 24), communicator=comm)
Hnospin.set_calculator(calc)
e_Hnospin = Hnospin.get_potential_energy()
niter_Hnospin = calc.get_number_of_iterations()
energies, sweight = raw_orbital_LDOS(calc, a=0, spin=0, angular='s')
energies, pdfweight = raw_orbital_LDOS(calc, a=0, spin=0, angular='pdf')

calc = GPAW(gpts=(24, 24, 24), occupations=FermiDirac(width=0, fixmagmom=True),
            poissonsolver=PoissonSolver('fd'),
            hund=True, communicator=comm)
Hspin.set_calculator(calc)
e_Hspin = Hspin.get_potential_energy()
niter_Hspin = calc.get_number_of_iterations()
energies, sweight_spin = raw_orbital_LDOS(calc, a=0, spin=0, angular='s')

calc = GPAW(gpts=(32, 32, 40), nbands=2,
            poissonsolver=PoissonSolver('fd'),
            communicator=comm)
LiH.set_calculator(calc)
e_LiH = LiH.get_potential_energy()
niter_LiH = calc.get_number_of_iterations()
energies, Li_orbitalweight = raw_orbital_LDOS(calc, a=0, spin=0, angular=None)
energies, H_orbitalweight = raw_orbital_LDOS(calc, a=1, spin=0, angular=None)
energies, Li_wzweight = raw_wignerseitz_LDOS(calc, a=0, spin=0)
energies, H_wzweight = raw_wignerseitz_LDOS(calc, a=1, spin=0)
n_a = calc.get_wigner_seitz_densities(spin=0)

print(sweight, pdfweight)
print(sweight_spin)
print(Li_wzweight)
print(H_wzweight)
print(n_a)

equal(sweight[0], 1.0, 0.06)
equal(pdfweight[0], 0.0, 0.0001)
equal(sweight_spin[0], 1.14, 0.06)
assert ((Li_wzweight - [.13, 0.93]).round(2) == 0).all()
assert ((H_wzweight - [0.87, 0.07]).round(2) == 0).all()
assert ((Li_wzweight + H_wzweight).round(5) == 1).all()
equal(n_a.sum(), 0.0, 1e-5)
equal(n_a[1], 0.737, 0.001)

print(Li_orbitalweight)
print(H_orbitalweight)
#               HOMO    s   py  pz  px  *s
Li_orbitalweight[0] -= [.5, .0, .6, .0, .0]
H_orbitalweight[0] -= [.7, .0, .0, .0, .0]

#              LUMO       s  py   pz  px  *s
Li_orbitalweight[1] -= [1.0, .0, 0.9, .0, .0]
H_orbitalweight[1] -= [0.1, .0, 0.0, .0, .0]

assert not Li_orbitalweight.round(1).any()
assert not H_orbitalweight.round(1).any()

ldos = RawLDOS(calc)
fname = 'ldbe.dat'
ldos.by_element_to_file(fname, shift=False)
ldos.by_element_to_file(fname, 2.0, shift=False)

energy_tolerance = 0.001
niter_tolerance = 0
equal(e_Hnospin, 0.153991, energy_tolerance)
equal(e_Hspin, -0.782309, energy_tolerance)
equal(e_LiH, -3.74582, energy_tolerance)



from ase.build import bulk
from gpaw import GPAW, Mixer, PW

atoms0 = bulk('Si')
atoms0.rattle(stdev=0.01, seed=17)  # Break symmetry

results = []

for mode in ['fd', 'pw']:
    for method in [None, 'paw', 'lcao']:
        atoms = atoms0.copy()

        kwargs = {}
        if mode == 'pw':
            if method == 'lcao':
                continue  # Not implemented yet
            kwargs.update(mode=PW(400.0))

        calc = GPAW(mixer=Mixer(0.4, 5, 20.0),
                    basis='dzp' if method == 'lcao' else {},
                    experimental={'reuse_wfs_method': method},
                    xc='oldLDA',
                    kpts=[2, 2, 2],
                    **kwargs)
        atoms.calc = calc
        E1 = atoms.get_potential_energy()
        niter1 = calc.scf.niter
        atoms.rattle(stdev=0.05)
        E2 = atoms.get_potential_energy()
        niter2 = calc.scf.niter
        results.append([mode, method, E1, E2, niter1, niter2])


if calc.wfs.world.rank == 0:
    for result in results:
        print(' '.join(str(x) for x in result))



from __future__ import print_function
from ase import Atoms
from ase.calculators.test import numeric_force
from gpaw import GPAW, FermiDirac, PoissonSolver
from gpaw.test import equal
from gpaw.xc.tools import vxc
a = 5.404
bulk = Atoms(symbols='Si8',
             positions=[(0, 0, 0.1 / a),
                        (0, 0.5, 0.5),
                        (0.5, 0, 0.5),
                        (0.5, 0.5, 0),
                        (0.25, 0.25, 0.25),
                        (0.25, 0.75, 0.75),
                        (0.75, 0.25, 0.75),
                        (0.75, 0.75, 0.25)],
             pbc=True)
bulk.set_cell((a, a, a), scale_atoms=True)
n = 20
calc = GPAW(gpts=(n, n, n),
            nbands='150%',
            occupations=FermiDirac(width=0.01),
            poissonsolver=PoissonSolver('fd', nn='M', relax='J'),
            kpts=(2, 2, 2),
            convergence={'energy': 1e-7}
            )
bulk.set_calculator(calc)
f1 = bulk.get_forces()[0, 2]
e1 = bulk.get_potential_energy()
v_xc = vxc(calc)
print(v_xc)
niter1 = calc.get_number_of_iterations()

f2 = numeric_force(bulk, 0, 2)
print((f1, f2, f1 - f2))
equal(f1, f2, 0.005)

# Volume per atom:
vol = a**3 / 8
de = calc.get_electrostatic_corrections() / vol
print(de)
assert abs(de[0] - -2.190) < 0.001

print((e1, f1, niter1))
energy_tolerance = 0.0025
force_tolerance = 0.01
niter_tolerance = 0
equal(e1, -46.6628, energy_tolerance)  # svnversion 5252
equal(f1, -1.38242356123, force_tolerance)  # svnversion 5252



from math import sqrt
import numpy as np

from gpaw.symmetry import Symmetry
from ase.dft.kpoints import monkhorst_pack

# Primitive diamond lattice, with Si lattice parameter
a = 5.475
cell_cv = .5 * a * np.array([(1, 1, 0), (1, 0, 1), (0, 1, 1)])
spos_ac = np.array([(.00, .00, .00),
                    (.25, .25, .25)])
id_a = [1, 1] # Two identical atoms
pbc_c = np.ones(3, bool)
bzk_kc = monkhorst_pack((4, 4, 4))

# Do check
symm = Symmetry(id_a, cell_cv, pbc_c, symmorphic=False)
symm.analyze(spos_ac)
ibzk_kc, w_k = symm.reduce(bzk_kc)[:2]
assert len(symm.op_scc) == 48
assert len(w_k) == 10
a = 3 / 32.
b = 1 / 32.
c = 6 / 32.
assert np.all(w_k == [a, b, a, c, c, a, a, a, a, b])
assert not symm.op_scc.sum(0).any()

# Rotate unit cell and check again:
cell_cv = a / sqrt(2) * np.array([(1, 0, 0),
                                  (0.5, sqrt(3) / 2, 0),
                                  (0.5, sqrt(3) / 6, sqrt(2.0 / 3))])
symm = Symmetry(id_a, cell_cv, pbc_c, symmorphic=False)
symm.analyze(spos_ac)
ibzkb_kc, wb_k = symm.reduce(bzk_kc)[:2]
assert len(symm.op_scc) == 48
assert abs(w_k - wb_k).sum() < 1e-14
assert abs(ibzk_kc - ibzkb_kc).sum() < 1e-14
assert not symm.op_scc.sum(0).any()

bzk_kc = monkhorst_pack((3, 3, 3))
symm = Symmetry(id_a, cell_cv, pbc_c)
symm.analyze(spos_ac)
ibzk_kc, w_k = symm.reduce(bzk_kc)[:2]
assert len(symm.op_scc) == 24
assert len(w_k) == 4
assert abs(w_k * 27 - (1, 12, 6, 8)).sum() < 1e-14
assert not symm.op_scc.sum(0).any()

# Rocksalt Ni2O2
a = 7.92
x = 2. * np.sqrt(1./3.)
y = np.sqrt(1./8.)
z1 = np.sqrt(1./24.)
z2 = np.sqrt(1./6.)
cell_cv = a * np.array([(x, y, -z1), (x, -y, -z1), (x, 0., z2)])
spos_ac = np.array([[0., 0. ,0.], [1./2., 1./2., 1./2.], [1./4., 1./4., 1./4.], [3./4., 3./4., 3./4.]])
id_a = [1, 2, 3, 3]
pbc_c = np.array([1, 1, 1], bool)
bzk_kc = monkhorst_pack((2, 2, 2))

# Do check
symm = Symmetry(id_a, cell_cv, pbc_c, symmorphic=False)
symm.analyze(spos_ac)
ibzk_kc, w_k = symm.reduce(bzk_kc)[:2]
assert len(symm.op_scc) == 12
assert len(w_k) == 2
assert np.all(w_k == [3/4., 1/4.])

#AlF3
a = 2.465250000
b = 1.423312751
c = 4.148733333
a1 = np.array([     a,-1.0*b, c])
a2 = np.array([ 0.0  , 2.0*b, c])
a3 = np.array([-1.0*a,-1.0*b, c])
cell_cv = np.array([a1,a2,a3])
id_a = [1, 1, 2, 2, 2, 2, 2, 2]
spos_ac = np.array([(0.000000000,   0.000000000,   0.000000000), (0.500000000,   0.500000000,   0.500000000),
                    (0.647758346,   0.852241654,   0.250000000), (0.852241654,   0.250000000,   0.647758346),
                    (0.250000000,   0.647758346,   0.852241654), (0.352241654,   0.147758346,   0.750000000),
                    (0.147758346,   0.750000000,   0.352241654), (0.750000000,   0.352241654,   0.147758346)])
pbc_c = np.array([1, 1, 1], bool)
bzk_kc = monkhorst_pack((3, 3, 3))

# Do check
symm = Symmetry(id_a, cell_cv, pbc_c, symmorphic=False)
symm.analyze(spos_ac)
ibzk_kc, w_k = symm.reduce(bzk_kc)[:2]
assert len(symm.op_scc) == 12
assert len(w_k) == 6
assert np.all(w_k == [1/27., 2/9., 2/9., 2/9., 2/9., 2/27.])
assert not symm.op_scc.sum(0).any()



"""Time dependent density functional perturbation theory."""
from __future__ import print_function
from gpaw.analyse.observers import Observer
from gpaw import GPAW
import ase.io
from ase.units import Hartree
from gpaw.mpi import world
import numpy as np


def transform_local_operator(gpw_file=None, tdop_file=None, fqop_file=None,
                             omega=None, eta=0.1, write_cube=False,
                             cube_scale=1e5):
    assert world.size == 1
    assert gpw_file is not None
    assert tdop_file is not None
    assert fqop_file is not None
    assert omega is not None
    omega /= Hartree
    eta /= Hartree

    tdf = open(tdop_file + '.sG', 'r')
    tdaf = open(tdop_file + '.asp', 'r')
    calc = GPAW(gpw_file)

    Fq_G = calc.hamiltonian.gd.zeros(dtype=complex)
    NG = np.prod(Fq_G.shape)

    # Reference values, taken from t=0 value
    Fq_dH_asp = {}
    ref_dH_asp = {}
    for a, dH_sp in calc.hamiltonian.dH_asp.items():
        ref_dH_asp[a] = np.zeros_like(dH_sp, dtype=complex)
        Fq_dH_asp[a] = np.zeros_like(dH_sp, dtype=complex)

    ref_sG = None

    iteration = 0

    while True:
        # Read pseudo potential and spherical corrections
        Td_G = np.fromfile(tdf, dtype=np.float64, count=NG)
        if np.prod(Td_G.shape) < NG:
            break

        Td_G = Td_G.reshape(Fq_G.shape)

        header = tdaf.readline().split()
        print(header)
        if len(header) == 0:
            break
        assert len(header) == 2
        time = float(header[0])
        natoms = int(header[1])

        Td_dH_asp = {}
        for n in range(natoms):
            data = tdaf.readline().split()
            a = int(data[0])
            Td_dH_asp[a] = np.zeros_like(calc.hamiltonian.dH_asp[a])
            Td_dH_asp[a][:] = [float(x) for x in data[1:]]

        if ref_sG is None:
            ref_sG = Td_G.copy()
            ref_dH_asp = {}
            for a, dH_sp in Td_dH_asp.items():
                ref_dH_asp[a] = Td_dH_asp[a].copy()

        if iteration == 0:
            print(time)
            assert time == 0.0
        c = np.exp(1j * time * omega - eta * time)
        print('Iteration', iteration, c)
        # Add to fourier transform
        Fq_G += c * (Td_G - ref_sG)
        for a, dH_sp in ref_dH_asp.items():
            Fq_dH_asp[a] += c * (Td_dH_asp[a] - ref_dH_asp[a])
        iteration += 1

    tdf.close()
    tdaf.close()

    fqf = open(fqop_file + '.sG', 'w')
    Fq_G.tofile(fqf)
    fqf.close()

    # Make sure we know how to read it
    fqf = open(fqop_file + '.sG', 'r')
    Fq2_G = np.fromfile(fqf, dtype=np.complex128, count=NG).reshape(Fq_G.shape)
    fqf.close()
    assert np.abs(Fq2_G - Fq_G).sum() < 1e-10  # XXX

    # Output the Fourier transformed Hamiltonian
    fqaf = open(fqop_file + '.asp', 'w')
    print('%.10f %.10f %d' % (omega, eta, len(Fq_dH_asp)), file=fqaf)
    for a, dH_sp in Fq_dH_asp.items():
        print(a, end=' ', file=fqaf)
        for H in dH_sp.ravel():
            print(H.real, H.imag, end=' ', file=fqaf)
        print(file=fqaf)

    ase.io.write(fqop_file + '.real.cube', calc.atoms,
                 data=cube_scale * Fq_G.real)
    ase.io.write(fqop_file + '.imag.cube', calc.atoms,
                 data=cube_scale * Fq_G.imag)


class TDDFPT(GPAW):
    def __init__(self, gpw_filename, Vop_filename, **kwargs):
        GPAW.__init__(self, gpw_filename, **kwargs)

        # Read the complex smooth potential
        tdf = open(Vop_filename + '.sG', 'r')
        NG = np.prod(self.hamiltonian.vt_sG.shape)
        self.cvt_sG = np.fromfile(tdf, dtype=np.complex128, count=NG).reshape(self.hamiltonian.vt_sG.shape)

        # Read the complex PAW corrections
        tdaf = open(Vop_filename + '.asp', 'r')
        header = tdaf.readline().split()
        assert len(header) == 3
        self.omega = float(header[0])
        self.eta = float(header[1])
        # NA = int(header[2])

        print('TDDFPT')
        print('Omega: %.4f eV' % (self.omega * Hartree))
        print('Eta: %.4f eV' % (self.omega * Hartree))

        self.cH_asp = {}
        for a, dH_sp in self.hamiltonian.dH_asp.items():
            data = tdaf.readline().split()
            a = int(data[0])
            self.cH_asp[a] = np.zeros_like(dH_sp, dtype=complex)
            self.cH_asp[a][:] = np.array([float(x) for x in data[1::2]])
            self.cH_asp[a][:] += 1j * np.array([float(x) for x in data[2::2]])

    def calculate(self):
        self.initialize(self.atoms)
        self.set_positions(self.atoms)

        # Real part
        self.hamiltonian.vt_sG[:] = self.cvt_sG.real
        for a, dH_sp in self.hamiltonian.dH_asp.items():
            self.hamiltonian.dH_asp[a][:] = self.cH_asp[a].real

        print(self.hamiltonian.vt_sG, np.sum(self.hamiltonian.vt_sG))
        Hp_MM = self.wfs.eigensolver.calculate_hamiltonian_matrix(
            self.hamiltonian, self.wfs, self.wfs.kpt_u[0], add_kinetic=False, root=-1)

        # Imag part
        self.hamiltonian.vt_sG[:] = self.cvt_sG.imag
        for a, dH_sp in self.hamiltonian.dH_asp.items():
            self.hamiltonian.dH_asp[a][:] = self.cH_asp[a].imag
        Hm_MM = self.wfs.eigensolver.calculate_hamiltonian_matrix(
            self.hamiltonian, self.wfs, self.wfs.kpt_u[0], add_kinetic=False, root=-1)

        S_MM = self.wfs.S_qMM[0]
        C0_nM = self.wfs.kpt_u[0].C_nM.copy()

        d_m = np.dot(np.dot(S_MM, Hm_MM), S_MM.T)
        Hm_nn = np.dot(np.dot(C0_nM, d_m), C0_nM.T.conjugate())
        print('Hm_nn', Hm_nn)

        d_p = np.dot(np.dot(S_MM, Hp_MM), S_MM.T)
        Hp_nn = np.dot(np.dot(C0_nM, d_p), C0_nM.T.conjugate())
        print('Hp_nn', Hp_nn)


class HamiltonianCollector(Observer):

    def __init__(self, filename, lcao):
        Observer.__init__(self)
        self.lcao = lcao
        self.filename = filename + '.sG'
        self.H_asp_filename = filename + '.asp'
        self.first_iteration = True

    def update(self):
        hamiltonian = self.lcao.hamiltonian

        if self.first_iteration:
            self.first_iteration = False
            if hamiltonian.world.rank == 0:
                # Create an empty file
                f = open(self.filename, 'w')
                f.close()
                f = open(self.H_asp_filename, 'w')
                f.close()

        vt_sG = hamiltonian.gd.collect(hamiltonian.vt_sG, broadcast=False)

        if hamiltonian.world.rank == 0:
            f = open(self.filename, 'a+')
            vt_sG.tofile(f)
            f.close()

        dH_asp = hamiltonian.dH_asp.copy()
        serial_partition = dH_asp.partition.as_serial()
        dH_asp.redistribute(serial_partition)

        if serial_partition.comm.rank == 0 and self.lcao.wfs.bd.comm.rank == 0:
            f = open(self.H_asp_filename, 'a+')
            print(self.lcao.time, len(dH_asp), file=f)
            for a, dH_sp in dH_asp.items():
                print(a, end=' ', file=f)
                for dH in dH_sp.ravel():
                    print(dH, end=' ', file=f)
                print(file=f)
            f.close()


class DensityCollector(Observer):

    def __init__(self, filename, lcao):
        Observer.__init__(self)
        self.lcao = lcao
        self.filename = filename + '.sG'
        self.D_asp_filename = filename + '.asp'
        self.first_iteration = True

    def update(self):
        hamiltonian = self.lcao.hamiltonian
        density = self.lcao.density

        if self.first_iteration:
            self.first_iteration = False
            if hamiltonian.world.rank == 0:
                # Create an empty file
                f = open(self.filename, 'w')
                f.close()
                f = open(self.D_asp_filename, 'w')
                f.close()

        nt_sG = density.gd.collect(density.nt_sG, broadcast=False)

        if hamiltonian.world.rank == 0:
            f = open(self.filename, 'a+')
            nt_sG.tofile(f)
            print(nt_sG.sum())
            f.close()

        D_asp = density.D_asp.copy()
        serial_partition = D_asp.partition.as_serial()
        D_asp.redistribute(serial_partition)

        if serial_partition.comm.rank == 0 and self.lcao.wfs.bd.comm.rank == 0:
            f = open(self.D_asp_filename, 'a+')
            print(self.lcao.time, len(D_asp), file=f)
            for a, D_sp in D_asp.items():
                print(a, end=' ', file=f)
                for D in D_sp.ravel():
                    print(D, end=' ', file=f)
                print(file=f)
            f.close()


"""
# Simplest example of use of LCAO-TDDFPT code

from ase import Atoms
from gpaw import GPAW
from ase.optimize import BFGS
from gpaw.tddft import photoabsorption_spectrum
from gpaw import PoissonSolver
from gpaw.lcaotddft.tddfpt import TDDFPT, HamiltonianCollector, transform_local_operator

# Sodium dimer
atoms = Atoms('H2', positions=[[0.0,0.0,0.0],[0.7,0.0,0.0]])
atoms.center(vacuum=2.5)

from gpaw.lcaotddft import LCAOTDDFT

# Increase accuragy of density for ground state
convergence = {'density':1e-7}

# Increase accuracy of Poisson Solver and apply multipole corrections up to l=2
poissonsolver = PoissonSolver(eps=1e-20, remove_moment=1+3+5)

if 1:
    # Calculate all bands
    td_calc = LCAOTDDFT(basis='sz', xc='oldLDA', h=0.3,
                        convergence=convergence, poissonsolver=poissonsolver)

    atoms.set_calculator(td_calc)
    atoms.get_potential_energy()
    td_calc.write('H2_gs.gpw',mode='all')

if 1:
    td_calc = LCAOTDDFT('H2_gs.gpw', poissonsolver=poissonsolver)
    td_calc.attach(HamiltonianCollector('H2.TdHam', td_calc))
    td_calc.kick([1e-5, 0.0, 0.0])
    td_calc.propagate(20, 100, 'H2.dm')

photoabsorption_spectrum('H2.dm','H2.spec', width=0.4)

transform_local_operator(gpw_file='H2_gs.gpw', tdop_file='H2.TdHam', fqop_file='H2.FqHam', omega=19.15, eta=1)
tddftpt = TDDFPT('H2_gs.gpw', 'H2.FqHam')
tddftpt.calculate()

"""



import numpy as np
from gpaw.mpi import world
from gpaw.utilities.partition import AtomPartition
from gpaw.arraydict import ArrayDict

gen = np.random.RandomState(0)


def shape(a):
    return (a, a // 2)  # Shapes: (0, 0), (1, 0), (2, 1), ...

natoms = 33

if world.size == 1:
    rank_a = np.zeros(natoms, int)
else:
    # When on more than 2 cores, make sure that at least one core
    # (rank=0) has zero entries:
    lower = 0 if world.size == 2 else 1
    rank_a = gen.randint(lower, world.size, natoms)
assert (rank_a < world.size).all()

serial = AtomPartition(world, np.zeros(natoms, int))
partition = AtomPartition(world, rank_a)
even_partition = partition.as_even_partition()


def check(atomdict, title):
    if world.rank == world.size // 2 or world.rank == 0:
        print('rank %d %s: %s' % (world.rank, title.rjust(10), atomdict))

    # Create a normal, "well-behaved" dict against which to test arraydict.
    ref = dict(atomdict)
    #print atomdict
    assert set(atomdict.keys()) == set(ref.keys())  # check keys()
    for a in atomdict:  # check __iter__, __getitem__
        #print ref[a].shape, atomdict[a].shape #ref[a].shape, atomdict[a].shape
        #print ref[a], atomdict[a]
        #assert 1#ref[a] is atomdict[a]
        assert ref[a] is atomdict[a]
    values = atomdict.values()
    for i, key in enumerate(atomdict):
        # AtomDict guarantees fixed ordering of keys.  Check that
        # values() ordering is consistent with loop ordering:
        assert values[i] is atomdict[key]

    items = list(atomdict.items())

    for i, (key, item) in enumerate(atomdict.items()):
        assert item is atomdict[key]
        assert item is ref[key]
        assert items[i][0] == key
        assert items[i][1] is item

    # Hopefully this should verify all the complicated stuff

ad = ArrayDict(partition, shape, float)
for key in ad:
    ad[key][:] = key
array0 = ad.toarray()

ad0 = dict(ad)
check(ad, 'new')
ad.redistribute(even_partition)
array1 = ad.toarray()
if world.rank > 1:
    assert array1.shape != array0.shape
check(ad, 'even')
ad.redistribute(serial)
check(ad, 'serial')
ad.redistribute(partition)
check(ad, 'back')

array2 = ad.toarray()
assert (array0 == array2).all()



"""Test Tran Blaha potential."""
from ase.dft.bandgap import bandgap
from ase.build import bulk
from gpaw import GPAW, PW, Mixer

def xc(name):
    return {'name': name, 'stencil': 1}

k = 8
atoms = bulk('Si')
atoms.calc = GPAW(mode=PW(300),
                  mixer=Mixer(0.8, 10, 50.0),
                  kpts={'size': (k, k, k), 'gamma': True},
                  xc=xc('TB09'),
                  convergence={'bands': -3},
                  txt='si.txt')
e = atoms.get_potential_energy()
gap, (sv, kv, nv), (sc, kc, nc) = bandgap(atoms.calc)
c = atoms.calc.hamiltonian.xc.c
print(gap, kv, kc)
print('c:', c)
assert abs(gap - 1.226) < 0.01
assert kv == 0 and kc == 24
assert abs(c - 1.136) < 0.01



from math import sqrt
from ase import Atoms
from gpaw import GPAW, FermiDirac

a = 2.4437
# Set up graphene structure
vacuum = 3
atoms = Atoms(symbols='C2',
              positions=[(0.5 * a, -sqrt(3) / 6 * a, vacuum),
                         (0.5 * a, +sqrt(3) / 6 * a, vacuum)],
              cell=[(0.5 * a, -0.5 * 3**0.5 * a, 0),
                    (0.5 * a, +0.5 * 3**0.5 * a, 0),
                    (0.0, 0.0, 2 * vacuum)])
atoms.pbc = True

# Gamma-point calculation of graphene
calc = GPAW(h=0.2,
            occupations=FermiDirac(width=0.15),
            kpts=(1, 1, 1),
            xc='LDA')
atoms.set_calculator(calc)
atoms.get_potential_energy()


kpts = [(1 / 2.0, 1 / 3.0, 0)]

# Calculate one K-point with symmetry:
calc.set(kpts=kpts, fixdensity=True)
calc.get_potential_energy()
eigs_True = calc.get_eigenvalues(kpt=0)

# Redo with the same K-point without symmetry:
calc.set(kpts=kpts,
         symmetry='off',
         fixdensity=True)
calc.get_potential_energy()
eigs_False = calc.get_eigenvalues(kpt=0)

assert abs(eigs_True[0] - eigs_False[0]) < 1e-4



from gpaw.atom.atompaw import AtomPAW
from gpaw.upf import UPFSetupData
import gpaw.test.pseudopotential.H_pz_hgh as H_hgh
import gpaw.test.pseudopotential.O_pz_hgh as O_hgh
from gpaw.mpi import world


def check(title, calc, epsref_n, threshold):
    eps_n = calc.wfs.kpt_u[0].eps_n
    print(title)
    for i, epsref in enumerate(epsref_n):
        err = abs(epsref - eps_n[i])
        ok = (err <= threshold)
        status = 'ok' if ok else 'FAILED'
        print('state %d | eps=%f | ref=%f | err=%f | tol=%s | %s' % (i,
                                                                     eps_n[i],
                                                                     epsref,
                                                                     err,
                                                                     threshold,
                                                                     status))
        assert ok


# Load pseudopotential from Python module as string, then
# write the string to a file, then load the file.
def upf(upf_module, fname):
    with open(fname, 'w') as fd:
        fd.write(upf_module.ps_txt)
    return UPFSetupData(fname)


if world.rank == 0:  # This test is not really parallel
    kwargs = dict(txt=None)
    for setup in ['paw', 'hgh', upf(H_hgh, 'H.pz-hgh.UPF')]:
        calc = AtomPAW('H', [[[1]]],
                       rcut=12.0, h=0.05,
                       setups={'H': setup}, **kwargs)
        tol = 5e-4 if setup in ['paw', 'hgh'] else 1e-3 # horrible UPF right now
        check('H %s' % setup, calc, [-0.233471], tol)


    for setup in ['paw', 'hgh', upf(O_hgh, 'O.pz-hgh.UPF')]:
        calc = AtomPAW('O', [[[2], [4]]],
                       rcut=10.0, h=0.035,
                       setups={'O': setup}, **kwargs)
        tol = 1e-3 if setup in ['paw', 'hgh'] else 5e-3 # horrible UPF right now
        check('O %s' % setup, calc, [-0.871362, -0.338381], tol)



from __future__ import print_function
from ase import Atom, Atoms
from gpaw import GPAW
from gpaw.test import equal
import numpy as np

bulk = Atoms([Atom('Li')], pbc=True)
k = 4
g = 8
calc = GPAW(gpts=(g, g, g), kpts=(k, k, k), nbands=2)#, txt=None)
bulk.set_calculator(calc)
a = np.linspace(2.6, 2.8, 5)
e = []
for x in a:
    bulk.set_cell((x, x, x))
    e1 = bulk.get_potential_energy()
    niter1 = calc.get_number_of_iterations()
    e.append(e1)

try:
    from gpaw.io.etsf import ETSFWriter
except ImportError:
    pass  # Scientific.IO.NetCDF was not installed
else:
    if calc.wfs.world.size == 1:
        ETSFWriter().write(calc)

fit = np.polyfit(a, e, 2)
a0 = np.roots(np.polyder(fit, 1))[0]
e0 = np.polyval(fit, a0)
print('a,e =', a0, e0)
equal(a0, 2.641, 0.001)
equal(e0, -1.98357, 0.0002)

energy_tolerance = 0.0002
equal(e1, -1.96157, energy_tolerance)



from __future__ import print_function
from ase.build import molecule

from gpaw import GPAW
from gpaw.cluster import Cluster
from gpaw.analyse.overlap import Overlap
import gpaw.solvation as solv
from gpaw.lrtddft import LrTDDFT
from gpaw.poisson import PoissonSolver

"""Check whether LrTDDFT in solvation works"""

h = 0.4
box = 2
nbands = 2
txt = '-'
txt = None

H2 = Cluster(molecule('H2'))
H2.minimal_box(box, h)

c1 = GPAW(h=h, txt=None, nbands=nbands)
c1.calculate(H2)

c2 = solv.SolvationGPAW(h=h,
                        txt=None,
                        nbands=nbands + 1,
                        **solv.get_HW14_water_kwargs())
c2.calculate(H2)
for poiss in [None, PoissonSolver(nn=c2.hamiltonian.poisson.nn)]:
    lr = LrTDDFT(c2, poisson=poiss)
    print(lr)
print(Overlap(c1).pseudo(c2))






from __future__ import print_function
from gpaw import GPAW, restart, FermiDirac
from ase import Atoms
from gpaw.test import equal, gen
import os
from gpaw.mpi import world

gen('Si', xcname='GLLBSC')

e = {}

energy_tolerance = 0.001

e_ref = {'LDA': {'restart': -5.583306128278814},
         'GLLBSC': {'restart': -5.458520154765278}}

for xc in ['LDA', 'GLLBSC']:
    a = 4.23
    bulk = Atoms('Si2',
                 cell=(a, a, a),
                 pbc=True,
                 scaled_positions=[[0, 0, 0], [0.5, 0.5, 0.5]])
    calc = GPAW(h=0.25,
                nbands=8,
                occupations=FermiDirac(width=0.01),
                kpts=(3, 3, 3),
                convergence={'eigenstates': 9.2e-11,
                             'bands': 8},
                xc=xc,
                eigensolver='cg')

    bulk.set_calculator(calc)
    e[xc] = {'direct': bulk.get_potential_energy()}
    print(calc.get_ibz_k_points())
    old_eigs = calc.get_eigenvalues(kpt=3)
    calc.write('Si_gs.gpw')
    del bulk
    del calc
    bulk, calc = restart('Si_gs.gpw',
                         fixdensity=True,
                         kpts=[[0, 0, 0], [1.0 / 3, 1.0 / 3, 1.0 / 3]])
    e[xc] = {'restart': bulk.get_potential_energy()}

    if world.rank == 0:
        os.remove('Si_gs.gpw')
    diff = calc.get_eigenvalues(kpt=1)[:6] - old_eigs[:6]
    if world.rank == 0:
        print("occ. eig. diff.", diff)
        error = max(abs(diff))
        assert error < 5e-6

    for mode in e[xc].keys():
        equal(e[xc][mode], e_ref[xc][mode], energy_tolerance)



from __future__ import print_function
from ase.build import molecule
from gpaw import GPAW, PW
from gpaw.test import equal
from gpaw.xc.rpa import RPACorrelation
from gpaw.xc.exx import EXX

ecut = 25

N2 = molecule('N2')
N2.center(vacuum=2.0)

calc = GPAW(mode=PW(force_complex_dtype=True),
            xc='PBE',
            parallel={'domain': 1},
            eigensolver='rmm-diis')
N2.set_calculator(calc)
E_n2_pbe = N2.get_potential_energy()

calc.diagonalize_full_hamiltonian(nbands=104, scalapack=True)
calc.write('N2.gpw', mode='all')

exx = EXX('N2.gpw')
exx.calculate()
E_n2_hf = exx.get_total_energy()

rpa = RPACorrelation('N2.gpw', nfrequencies=8)
E_n2_rpa = rpa.calculate(ecut=[ecut])

N = molecule('N')
N.set_cell(N2.cell)

calc = GPAW(mode=PW(force_complex_dtype=True),
            xc='PBE',
            parallel={'domain': 1},
            eigensolver='rmm-diis')
N.set_calculator(calc)
E_n_pbe = N.get_potential_energy()

calc.diagonalize_full_hamiltonian(nbands=104, scalapack=True)
calc.write('N.gpw', mode='all')

exx = EXX('N.gpw')
exx.calculate()
E_n_hf = exx.get_total_energy()

rpa = RPACorrelation('N.gpw', nfrequencies=8)
E_n_rpa = rpa.calculate(ecut=[ecut])

print('Atomization energies:')
print('PBE: ', E_n2_pbe - 2 * E_n_pbe)
print('HF: ', E_n2_hf - 2 * E_n_hf)
print('HF+RPA: ', E_n2_hf - 2 * E_n_hf + E_n2_rpa[0] - 2 * E_n_rpa[0])

equal(E_n2_rpa - 2 * E_n_rpa, -1.68, 0.02)



import numpy as np

from gpaw import debug
from gpaw import mpi

"""
def dscf_find_lumo(paw,band):

    # http://trac.fysik.dtu.dk/projects/gpaw/browser/trunk/doc/documentation/dscf/lumo.py?format=raw

    assert band in [5,6]

    #Find band corresponding to lumo
    lumo = paw.get_pseudo_wave_function(band=band, kpt=0, spin=0)
    lumo = np.reshape(lumo, -1)

    wf1_k = [paw.get_pseudo_wave_function(band=5, kpt=k, spin=0) for k in range(paw.wfs.kd.nibzkpts)]
    wf2_k = [paw.get_pseudo_wave_function(band=6, kpt=k, spin=0) for k in range(paw.wfs.kd.nibzkpts)]

    band_k = []
    for k in range(paw.wfs.kd.nibzkpts):
        wf1 = np.reshape(wf1_k[k], -1)
        wf2 = np.reshape(wf2_k[k], -1)
        p1 = np.abs(np.dot(wf1, lumo))
        p2 = np.abs(np.dot(wf2, lumo))

        if p1 > p2:
            band_k.append(5)
        else:
            band_k.append(6)

    return band_k
"""

# -------------------------------------------------------------------

"""
def mpi_debug(text):
    if isinstance(text,list):
        for t in text:
            mpi_debug(t)
    else:
        print 'mpi.rank=%d, %s' % (mpi.rank,text)
"""

global msgcount
msgcount = 0

def mpi_debug(data, ordered=True):
    global msgcount

    if not isinstance(data, list):
        data = [data]

    if ordered:
        for i in range(mpi.rank):
            mpi.world.barrier()

    for txt in data:
        print('%02d-mpi%d, %s' % (msgcount, mpi.rank, txt))
        if ordered:
            msgcount += 1

    if ordered:
        for i in range(mpi.size-mpi.rank):
            mpi.world.barrier()

# -------------------------------------------------------------------

def dscf_find_atoms(atoms,symbol):
    chemsyms = atoms.get_chemical_symbols()
    return np.where([s==symbol for s in chemsyms])[0]

# -------------------------------------------------------------------

# Helper function in case of hs_operators having ngroups > 1
def SliceGen(psit_nG, operator):
    assert psit_nG.ndim == 4
    assert psit_nG.shape[0] == operator.bd.mynbands
    assert np.all(psit_nG.shape[1:] == operator.gd.n_c)
    assert operator.bd.mynbands % operator.nblocks == 0
    M = operator.bd.mynbands // operator.nblocks
    for j in range(operator.nblocks):
        n1 = j * M
        n2 = n1 + M
        yield psit_nG[n1:n2]
    raise StopIteration

from gpaw.kpoint import GlobalKPoint
from gpaw.hs_operators import MatrixOperator

def dscf_kpoint_overlaps(paw, phasemod=True, broadcast=True):
    bd = paw.wfs.bd
    gd = paw.wfs.gd
    kd = paw.wfs.kd
    operator = MatrixOperator(paw.wfs.orthoksl, hermitian=False)

    # Find the kpoint with lowest kpt.k_c (closest to gamma point)
    k0 = np.argmin(np.sum(paw.wfs.kd.ibzk_kc**2,axis=1)**0.5)

    # Maintain list of a single global reference kpoint for each spin
    kpt0_s = []
    for s0 in range(kd.nspins):
        q0 = k0 - kd.get_offset() % kd.nibzkpts
        kpt0 = GlobalKPoint(None, s0, k0, q0, None)
        kpt0.update(paw.wfs)
        kpt0_s.append(kpt0)

    if phasemod:
        # Scaled grid point positions used for exponential with ibzk_kc
        # cf. wavefunctions.py lines 90-91 rev 4500(ca)
        # phase_cd = np.exp(2j * np.pi * sdisp_cd * ibzk_kc[k, :, np.newaxis])
        r_cG = gd.empty(3)
        for c, r_G in enumerate(r_cG):
            slice_c2G = [np.newaxis, np.newaxis, np.newaxis]
            slice_c2G[c] = slice(None) #this means ':'
            r_G[:] = np.arange(gd.beg_c[c], gd.end_c[c], \
                               dtype=float)[slice_c2G] / gd.N_c[c]

    X_unn = np.empty((kd.mynks, bd.nbands, bd.nbands), dtype=paw.wfs.dtype)
    for myu, kpt in enumerate(paw.wfs.kpt_u):
        u = kd.global_index(myu)
        s, k = kd.what_is(u)
        kpt0 = kpt0_s[s]
        X_nn = X_unn[myu]

        if phasemod:
            assert paw.wfs.dtype == complex, 'Phase modification is complex!'

            k0_c = kd.ibzk_kc[k0]
            k_c = kd.ibzk_kc[k]
            eirk_G = np.exp(2j*np.pi*np.sum(r_cG*(k_c-k0_c)[:,np.newaxis,np.newaxis,np.newaxis], axis=0))
            psit0_nG = eirk_G[np.newaxis,...]*kpt0.psit_nG

            P0_ani = paw.wfs.pt.dict(bd.mynbands)
            for a, P0_ni in P0_ani.items():
                # Expanding the exponential exp(ikr)=exp(ikR)*exp(ik(r-R))
                # and neglecting the changed P_ani integral exp(ik(r-R))~1
                P0_ni[:] = np.exp(2j*np.pi*np.sum(paw.spos_ac[a]*(k_c-k0_c), axis=0)) * kpt0.P_ani[a]

            ## NB: No exp(ikr) approximate here, but has a parallelization bug
            #kpt0_rank, myu0 = kd.get_rank_and_index(kpt0.s, kpt0.k)
            #if kd.comm.rank == kpt0_rank:
            #    paw.wfs.pt.integrate(psit0_nG, P0_ani, kpt0.q)
            #for a, P0_ni in P0_ani.items():
            #    kd.comm.broadcast(P0_ni, kpt0_rank)
        else:
            psit0_nG = kpt0.psit_nG
            P0_ani = kpt0.P_ani

        """
        if paw.wfs.world.size == 1:
            for n, psit_G in enumerate(kpt.psit_nG):
                for n0, psit0_G in enumerate(psit0_nG):
                    X_nn[n,n0] = np.vdot(psit_G, psit0_G)*gd.dv
            for a in range(len(paw.get_atoms())):
                P_ni, P0_ni, dO_ii = kpt.P_ani[a], P0_ani[a], paw.wfs.setups[a].dO_ii
                for n, P_i in enumerate(P_ni):
                    for n0, P0_i in enumerate(P0_ni):
                        X_nn[n,n0] += np.vdot(P_i, np.dot(dO_ii, P0_i))
        """
        X = lambda psit_nG, g=SliceGen(psit0_nG, operator): next(g)
        dX = lambda a, P_ni: np.dot(P0_ani[a], paw.wfs.setups[a].dO_ii)
        X_nn[:] = operator.calculate_matrix_elements(kpt.psit_nG, kpt.P_ani, X, dX).T

    if broadcast:
        if bd.comm.rank == 0:
            gd.comm.broadcast(X_unn, 0)
        bd.comm.broadcast(X_unn, 0)

    return kpt0_s, X_unn

# -------------------------------------------------------------------

def dscf_find_bands(paw,bands,data=None):
    """Entirely serial, but works regardless of parallelization. DOES NOT WORK WITH DOMAIN-DECOMPOSITION IN GPAW v0.5.2725 """ #TODO!

    raise DeprecationWarning('About to be replaced with something better.')

    if data is None:
        data = range(len(bands))
    else:
        assert len(data)==len(bands), 'Length mismatch.'

    k0 = 0 #TODO find kpt with lowest kpt.k_c (closest to gamma point)

    gamma_siG = []
    for s in range(paw.wfs.nspins):
        gamma_siG.append([paw.get_pseudo_wave_function(band=n,kpt=k0,spin=s).ravel() for n in bands]) #TODO! paw.get_pseudo fails with domain-decomposition from tar-file

    band_ui = []
    data_ui = []

    for u,kpt in enumerate(paw.wfs.kpt_u):
        band_i = []
        data_i = []

        for (i,n) in enumerate(bands):
            if kpt.k == k0:
                wf = gamma_siG[kpt.s][i]
            else:
                #wf = paw.get_pseudo_wave_function(n, kpt.k, kpt.s, pad=False).ravel()
                wf = paw.get_pseudo_wave_function(band=n,kpt=kpt.k,spin=kpt.s).ravel()

            overlaps = [np.abs(np.dot(wf,gamma_siG[kpt.s][i])) for i in range(len(bands))]
            if debug: mpi_debug('u=%d, i=%d, band=%d, overlaps=%s' % (u,i,n,str(overlaps)))
            p = np.argmax(overlaps)
            band_i.append(bands[p])
            data_i.append(data[p])

        assert len(np.unique(band_i))==len(np.sort(band_i)), 'Non-unique band range' #TODO!

        band_ui.append(band_i)
        data_ui.append(data_i)

    return (band_ui,data_ui,)

# -------------------------------------------------------------------

def dscf_linear_combination(paw, molecule, bands, coefficients):
    """Full parallelization over k-point - grid-decomposition parallelization needs heavy testing.""" #TODO!

    raise DeprecationWarning('About to be replaced with something better.')

    if debug: dumpkey = mpi.world.size == 1 and 'serial' or 'mpi'

    (band_ui,coeff_ui,) = dscf_find_bands(paw,bands,coefficients)

    if debug: mpi_debug('band_ui=%s, coeff_ui=%s' % (band_ui,coeff_ui))

    P_aui = {}
    for m,a in enumerate(molecule):
        if debug: mpi_debug('a=%d, paw.wfs.kd.nibzkpts=%d, len(paw.wfs.kpt_u)=%d, paw.wfs.setups[%d].ni=%d' % (a, paw.wfs.kd.nibzkpts, len(paw.wfs.kpt_u), a, paw.wfs.setups[a].ni))
        P_aui[m] = np.zeros((len(paw.wfs.kpt_u),paw.wfs.setups[a].ni),dtype=complex)

    for u,kpt in enumerate(paw.wfs.kpt_u):

        band_i = band_ui[u]
        coeff_i = coeff_ui[u]

        #if debug: mpi_debug(['paw.wfs.kpt_u[%d].P_ani[:,%d,:].shape=%s' % (u,n,str(paw.wfs.kpt_u[u].P_ani[:,n,:].shape)) for n in bands])

        for m,a in enumerate(molecule):
            """
            if debug:
                for n in bands:
                    print 'mpi.rank=%d, paw.nuclei[%d].P_uni[:,%d,:].shape=' % (mpi.rank,a,n), paw.nuclei[a].P_uni[:,n,:].shape

                print 'mpi.rank=%d, test.shape=' % mpi.rank, np.sum([c*paw.nuclei[a].P_uni[:,n,:] for (c,n) in zip(coefficients,bands)],axis=0).shape
            """

            #P_aui[m] += np.sum([c*paw.nuclei[a].P_uni[:,n,:] for (c,n) in zip(coefficients,bands)],axis=0)

            #if paw.nuclei[a].in_this_domain: #TODO what happened to this one in guc?
            if True:
                P_aui[m][u,:] += np.sum([c*kpt.P_ani[a][n,:] for (c,n) in zip(coeff_i,band_i)],axis=0)

                if debug: kpt.P_ani[a][:,:].dump('dscf_tool_P_ani_a%01d_k%01ds%01d_%s%02d.pickle' % (a,kpt.k,kpt.s,dumpkey,mpi.rank))

    #paw.wfs.gd.comm.sum(P_aui) #TODO HUH?!

    if debug: P_aui.dump('dscf_tool_P_aui_%s%02d.pickle' % (dumpkey,mpi.rank))

    """
    if debug and mpi.rank == 0:
        print 'P_aui.shape=',P_aui.shape

        for (a,P_ui) in enumerate(P_aui):
            print 'P_aui[%d].shape=' % a,P_ui.shape

        print 'P_aui=',P_aui

        print 'gd.Nc=',paw.wfs.gd.N_c
    """

    if debug: mpi_debug('P_aui.shape='+str(P_aui.shape))

    #wf_u = [np.sum([c*paw.wfs.kpt_u[u].psit_nG[n] for (c,n) in zip(coefficients,bands)],axis=0) for u in range(0,len(paw.wfs.kpt_u))]
    #wf_u = np.zeros((paw.wfs.kd.nibzkpts,paw.wfs.gd.N_c[0]-1,paw.wfs.gd.N_c[1]-1,paw.wfs.gd.N_c[2]-1))#,dtype=complex)
    wf_u = paw.wfs.gd.zeros(len(paw.wfs.kpt_u),dtype=complex)

    gd_slice = paw.wfs.gd.get_slice()

    if debug: mpi_debug('gd_slice='+str(gd_slice))

    for u,kpt in enumerate(paw.wfs.kpt_u):
        if debug: mpi_debug('u=%d, k=%d, s=%d, paw.wfs.kd.comm.rank=%d, paw.wfs.kd.comm.rank=%d, gd.shape=%s, psit.shape=%s' % (u, kpt.k, kpt.s, paw.wfs.kd.comm.rank, paw.wfs.kd.comm.rank, str(wf_u[0].shape), str(np.array(kpt.psit_nG[0])[gd_slice].shape)))

        #wf_u[u] += np.sum([c*np.array(kpt.psit_nG[n])[gd_slice] for (c,n) in zip(coefficients,bands)],axis=0)

        band_i = band_ui[u]
        coeff_i = coeff_ui[u]
        wf_u[u] += np.sum([c*np.array(kpt.psit_nG[n])[gd_slice] for (c,n) in zip(coeff_i,band_i)],axis=0)

    #paw.wfs.gd.comm.sum(wf_u)

    if debug: mpi_debug('|wf_u|^2=%s' % str([np.sum(np.abs(wf.flatten())**2) for wf in wf_u]))

    """
    if debug and mpi.rank == 0:
        print 'wf_u.shape=',wf_u.shape

        for (u,wf) in enumerate(wf_u):
            print 'wf[%d].shape=' % u,wf.shape

        for (u,wf) in enumerate(wf_u):
            print 'wf_u[%d].shape=' % u,wf.shape
    """

    return (P_aui,wf_u,)

# -------------------------------------------------------------------

def dscf_decompose_occupations(ft_mn):
    # We define a Hermitian matrix with elements f[m,n] = c[m]*c[n].conj()
    # and rewrite the coefficients in polar form as c[n] = r[n]*exp(1j*v[n]):
    # f[m,n] = r[m]*exp(1j*v[m])*r[n]*exp(-1j*v[n]) = r[m]*r[n]*exp(1j*(v[m]-v[n]))

    # Assuming c[i] is positive real for some i, i.e. v[i] = 0, then
    # c[m] = f[m,i]/r[i] = r[m]*exp(1j*v[m]) , where r[i] = sqrt(f[i,i])
    i = np.argmax(np.abs(ft_mn.diagonal()))
    c_n = ft_mn[:,i]/np.abs(ft_mn[i,i])**0.5

    if (np.abs(ft_mn-np.outer(c_n,c_n.conj()))>1e-12).any():
        raise RuntimeError('Hermitian matrix cannot be decomposed')

    # Note that c_n is only defined up to an arbitrary phase factor
    return c_n

# -------------------------------------------------------------------

from gpaw.utilities import unpack
from gpaw.utilities.tools import tri2full

def dscf_matrix_elements(paw, kpt, A, dA):
    operator = paw.wfs.overlap.operator
    A_nn = operator.calculate_matrix_elements(kpt.psit_nG, kpt.P_ani, \
        A, dA).T.copy() # transpose to get A_nn[m,n] = <m|A|n>
    tri2full(A_nn, 'U') # fill in from upper to lower...

    # Calculate <o|A|o'> = sum_nn' <o|n><n|A|n'><n'|o'> where c_on = <n|o>
    A_oo = np.dot(kpt.c_on.conj(), np.dot(A_nn, kpt.c_on.T))
    return A_oo, A_nn

def dscf_overlap_elements(paw, kpt):
    # Copy/paste from gpaw/overlap.py lines 83-86 rev. 4808
    operator = paw.wfs.overlap.operator
    assert operator.nblocks == 1

    S = lambda x: x
    dS_aii = dict([(a, paw.wfs.setups[a].dO_ii) for a in kpt.P_ani.keys()])
    return dscf_matrix_elements(paw, kpt, S, dS_aii)

def dscf_hamiltonian_elements(paw, kpt):
    # Copy/paste from gpaw/eigensolvers/eigensolver.py lines 155-170 rev. 4808
    operator = paw.wfs.overlap.operator
    assert operator.nblocks == 1
    Htpsit_xG = operator.suggest_temporary_buffer(kpt.psit_nG.dtype)

    def H(psit_xG):
        paw.wfs.kin.apply(psit_xG, Htpsit_xG, kpt.phase_cd)
        paw.hamiltonian.apply_local_potential(psit_xG, Htpsit_xG, kpt.s)
        paw.hamiltonian.xc.add_non_local_terms(psit_xG, Htpsit_xG, kpt)
        return Htpsit_xG

    dH_aii = dict([(a, unpack(dH_sp[kpt.s])) for a, dH_sp \
        in paw.hamiltonian.dH_asp.items()])

    H_oo, H_nn = dscf_matrix_elements(paw, kpt, H, dH_aii)
    eps_o = H_oo.real.diagonal()
    return eps_o, H_oo, H_nn

# -------------------------------------------------------------------

from ase.units import Hartree
from gpaw.io.tar import Writer, Reader, FileReference
from gpaw.occupations import FermiDirac
from gpaw.band_descriptor import BandDescriptor

def dscf_reconstruct_orbitals_k_point(paw, norbitals, mol, kpt):
    bd, gd = paw.wfs.bd, paw.wfs.gd
    if bd.comm.size != 1:
        raise NotImplementedError('Undefined action for band parallelization.')

    f_o = np.zeros(norbitals, dtype=float)
    eps_o = np.zeros(norbitals, dtype=float)
    wf_oG = gd.zeros(norbitals, dtype=complex)

    P_aoi = {}
    for a in mol:
        P_aoi[a] = np.zeros((norbitals,paw.wfs.setups[a].ni), dtype=complex)

    for o, c_n in enumerate(kpt.c_on):
        f_o[o] = np.dot(c_n.conj(), c_n)
        eps_o[o] = np.dot(np.abs(c_n)**2 / f_o[o], kpt.eps_n) # XXX use dscf_hamiltonian_elements for accuracy

        for n, psit_G in enumerate(np.asarray(kpt.psit_nG)):
            wf_oG[o] += c_n[n] / f_o[o]**0.5 * psit_G

        for a, P_oi in P_aoi.items():
            P_oi[o] += np.sum(c_n[:,np.newaxis] / f_o[o]**0.5 * kpt.P_ani[a], axis=0)

    return (f_o, eps_o, wf_oG, P_aoi,)


def dscf_save_band(filename, paw, n):
    """Extract and save all information for band `n` to a tar file."""
    world, gd, kd = paw.wfs.world, paw.wfs.gd, paw.wfs.kd
    if world.rank == 0:
        # Minimal amount of information needed:
        w = Writer(filename)
        w.dimension('nspins', kd.nspins)
        w.dimension('nibzkpts', kd.nibzkpts)
        w.dimension('nproj', sum([setup.ni for setup in paw.wfs.setups]))
        ng = gd.get_size_of_global_array()
        w.dimension('ngptsx', ng[0])
        w.dimension('ngptsy', ng[1])
        w.dimension('ngptsz', ng[2])

    # Write projections:
    if world.rank == 0:
        w.add('Projection', ('nspins', 'nibzkpts', 'nproj'), dtype=paw.wfs.dtype)
    for s in range(kd.nspins):
        for k in range(kd.nibzkpts):
            all_P_ni = paw.wfs.collect_projections(k, s) # gets all bands
            if world.rank == 0:
                w.fill(all_P_ni[n])

    # Write wave functions:
    if world.rank == 0:
        w.add('PseudoWaveFunction', ('nspins', 'nibzkpts',
                                     'ngptsx', 'ngptsy', 'ngptsz'),
              dtype=paw.wfs.dtype)
    for s in range(kd.nspins):
        for k in range(kd.nibzkpts):
            psit_G = paw.wfs.get_wave_function_array(n, k, s)
            if world.rank == 0:
                w.fill(psit_G)

    if world.rank == 0:
        # Close the file here to ensure that the last wave function is
        # written to disk:
        w.close()

    # We don't want the slaves to start reading before the master has
    # finished writing:
    world.barrier()


def dscf_load_band(filename, paw, molecule=None):
    """Load and distribute all information for a band from a tar file."""
    if not paw.wfs:
        paw.initialize()
    bd, gd, kd = paw.wfs.bd, paw.wfs.gd, paw.wfs.kd
    if bd.comm.size != 1:
        raise NotImplementedError('Undefined action for band parallelization.')

    r = Reader(filename)
    assert (r.dimension('nspins') == kd.nspins and \
        r.dimension('nibzkpts') == kd.nibzkpts), 'Incompatible spin/kpoints.'

    # Read wave function for every spin/kpoint owned by this rank
    psit_uG = gd.empty(kd.mynks, kd.dtype)
    for myu, psit_G in enumerate(psit_uG):
        u = kd.global_index(myu)
        s, k = kd.what_is(u)
        if gd.comm.rank == 0:
            big_psit_G = np.array(r.get('PseudoWaveFunction', s, k), paw.wfs.dtype)
        else:
            big_psit_G = None
        gd.distribute(big_psit_G, psit_G)

    # Find domain ranks for each atom
    atoms = paw.get_atoms()
    rank_a = gd.get_ranks_from_positions(paw.spos_ac)
    #my_atom_indices = np.argwhere(rank_a == gd.comm.rank).ravel()
    #assert np.all(my_atom_indices == paw.wfs.pt.my_atom_indices)
    assert r.dimension('nproj') == sum([setup.ni for setup in paw.wfs.setups])

    if molecule is None:
        molecule = range(len(atoms))

    # Read projections for every spin/kpoint and atom owned by this rank
    P_uai = [{}] * kd.mynks #[paw.wfs.pt.dict() for myu in range(kd.mynks)]
    for myu, P_ai in enumerate(P_uai):
        u = kd.global_index(myu)
        s, k = kd.what_is(u)
        P_i = r.get('Projection', s, k)
        i1 = 0
        for a in molecule:
            setup = paw.wfs.setups[a]
            i2 = i1 + setup.ni
            if gd.comm.rank == rank_a[a]:
                P_ai[a] = np.array(P_i[i1:i2], paw.wfs.dtype)
            i1 = i2

    return psit_uG, P_uai


def dscf_collapse_orbitals(paw, nbands_max='occupied', f_tol=1e-4,
                           verify_density=True, nt_tol=1e-5, D_tol=1e-3):

    bd, gd, kd = paw.wfs.bd, paw.wfs.gd, paw.wfs.kd
    if bd.comm.size != 1:
        raise NotImplementedError('Undefined action for band parallelization.')

    f_skn = np.empty((kd.nspins, kd.nibzkpts, bd.nbands), dtype=float)
    for s, f_kn in enumerate(f_skn):
        for k, f_n in enumerate(f_kn):
            kpt_rank, myu = kd.get_rank_and_index(s, k)
            if kd.comm.rank == kpt_rank:
                f_n[:] = paw.wfs.kpt_u[myu].f_n
            kd.comm.broadcast(f_n, kpt_rank)

    # Find smallest band index, from which all bands have negligeble occupations
    n0 = np.argmax(f_skn<f_tol, axis=-1).max()
    assert np.all(f_skn[...,n0:]<f_tol) # XXX use f_skn[...,n0:].sum()<f_tol

    # Read the number of Delta-SCF orbitals
    norbitals = paw.occupations.norbitals
    if debug: mpi_debug('n0=%d, norbitals=%d, bd:%d, gd:%d, kd:%d' % (n0,norbitals,bd.comm.size,gd.comm.size,kd.comm.size))

    if nbands_max < 0:
        nbands_max = n0 + norbitals - nbands_max
    elif nbands_max == 'occupied':
        nbands_max = n0 + norbitals

    assert nbands_max >= n0 + norbitals, 'Too few bands to include occupations.'
    ncut = nbands_max-norbitals

    if debug: mpi_debug('nbands_max=%d' % nbands_max)

    paw.wfs.initialize_wave_functions_from_restart_file() # hurts memmory

    for kpt in paw.wfs.kpt_u:
        mol = kpt.P_ani.keys() # XXX stupid
        (f_o, eps_o, wf_oG, P_aoi,) = dscf_reconstruct_orbitals_k_point(paw, norbitals, mol, kpt)

        assert np.abs(f_o-1).max() < 1e-9, 'Orbitals must be properly normalized.'
        f_o = kpt.ne_o # actual ocupatiion numbers

        # Crop band-data and inject data for Delta-SCF orbitals
        kpt.f_n = np.hstack((kpt.f_n[:n0], f_o, kpt.f_n[n0:ncut]))
        kpt.eps_n = np.hstack((kpt.eps_n[:n0], eps_o, kpt.eps_n[n0:ncut]))
        for a, P_ni in kpt.P_ani.items():
            kpt.P_ani[a] = np.vstack((P_ni[:n0], P_aoi[a], P_ni[n0:ncut]))

        old_psit_nG = kpt.psit_nG
        kpt.psit_nG = gd.empty(nbands_max, dtype=old_psit_nG.dtype)

        if isinstance(old_psit_nG, FileReference):
            assert old_psit_nG.shape[-3:] == wf_oG.shape[-3:], 'Shape mismatch!'

            # Read band-by-band to save memory as full psit_nG may be large
            for n,psit_G in enumerate(kpt.psit_nG):
                if n < n0:
                    full_psit_G = old_psit_nG[n]
                elif n in range(n0,n0+norbitals):
                    full_psit_G = wf_oG[n-n0]
                else:
                    full_psit_G = old_psit_nG[n-norbitals]
                gd.distribute(full_psit_G, psit_G)
        else:
            kpt.psit_nG[:n0] = old_psit_nG[:n0]
            kpt.psit_nG[n0:n0+norbitals] = wf_oG
            kpt.psit_nG[n0+norbitals:] = old_psit_nG[n0:ncut]

        del kpt.ne_o, kpt.c_on, old_psit_nG

    del paw.occupations.norbitals

    # Change various parameters related to new number of bands
    paw.wfs.bd = BandDescriptor(nbands_max, bd.comm, bd.strided)
    if paw.wfs.eigensolver:
        paw.wfs.eigensolver.initialized = False
    del bd

    # Crop convergence criteria nbands_converge to new number of bands
    par = paw.input_parameters
    if 'convergence' in par:
        cc = par['convergence']
        if 'bands' in cc:
            cc['bands'] = min(nbands_max, cc['bands'])

    # Replace occupations class with a fixed variant (gets the magmom right) XXX?!?
    fermilevel, magmom = paw.occupations.fermilevel, paw.occupations.magmom
    paw.occupations = FermiDirac(paw.occupations.width * Hartree, paw.occupations.fixmagmom)
    paw.occupations.fermilevel = fermilevel
    paw.occupations.magmom = magmom
    del fermilevel, magmom

    # For good measure, self-consistency information should be destroyed
    paw.scf.reset()

    if verify_density:
        paw.initialize_positions()

        # Re-calculate pseudo density and watch for changes
        old_nt_sG = paw.density.nt_sG.copy()
        paw.density.calculate_pseudo_density(paw.wfs)
        if debug: mpi_debug('delta-density: %g' % np.abs(old_nt_sG-paw.density.nt_sG).max())
        assert np.abs(paw.density.nt_sG-old_nt_sG).max() < nt_tol, 'Density changed!'

        # Re-calculate atomic density matrices and watch for changes
        old_D_asp = {}
        for a,D_sp in paw.density.D_asp.items():
            old_D_asp[a] = D_sp.copy()
        paw.wfs.calculate_atomic_density_matrices(paw.density.D_asp)
        if debug: mpi_debug('delta-D_asp: %g' % max([0]+[np.abs(D_sp-old_D_asp[a]).max() for a,D_sp in paw.density.D_asp.items()]))
        for a,D_sp in paw.density.D_asp.items():
            assert np.abs(D_sp-old_D_asp[a]).max() < D_tol, 'Atom %d changed!' % a




"""Test selfconsistent RSF calculation with Yukawa potential including vc."""
from ase import Atoms
from gpaw import GPAW, setup_paths, KohnShamConvergenceError
from gpaw.xc.hybrid import HybridXC
from gpaw.poisson import PoissonSolver
from gpaw.occupations import FermiDirac
from gpaw.test import gen
from gpaw.eigensolvers import RMMDIIS
from gpaw.cluster import Cluster

if setup_paths[0] != '.':
    setup_paths.insert(0, '.')

h = 0.3

# No energies - simpley convergence test, esp. for 3d TM

# for atom in ['F', 'Cl', 'Br', 'Cu', 'Ag']:
for atom in ['Ti']:
    gen(atom, xcname='PBE', scalarrel=False, exx=True)
    work_atom = Cluster(Atoms(atom, [(0, 0, 0)]))
    work_atom.minimal_box(4, h=h)
    work_atom.translate([0.01, 0.02, 0.03])
    work_atom.set_initial_magnetic_moments([2.0])
    calculator = GPAW(convergence={'energy': 0.01,
                                   'eigenstates': 3,
                                   'density': 3},
                      eigensolver=RMMDIIS(),
                      poissonsolver=PoissonSolver(use_charge_center=True),
                      occupations=FermiDirac(width=0.0, fixmagmom=True),
                      h=h, maxiter=35)   # Up to 24 are needed by now
    calculator.set(xc=HybridXC('PBE0'))
    calculator.set(txt=atom + '-PBE0.txt')
    work_atom.set_calculator(calculator)
    try:
        work_atom.get_potential_energy()
    except KohnShamConvergenceError:
        pass
    assert calculator.scf.converged, 'Calculation not converged'



from __future__ import print_function
from gpaw.atom.aeatom import AllElectronAtom, c
from gpaw.test import equal

Z = 79  # gold atom
kwargs = dict(ngpts=5000, alpha2=1000 * Z**2, ngauss=200)

# Test Schroedinger equation:
aea = AllElectronAtom(Z, log=None)
aea.initialize(**kwargs)

errors = []
for channel in aea.channels:
    channel.solve(-Z)
    for n in range(7):
        e = channel.e_n[n]
        e0 = -0.5 * Z**2 / (n + channel.l + 1)**2
        errors.append(abs(e / e0 - 1))
print(max(errors))
equal(max(errors), 0, 2.0e-5)

# Test Dirac equation:
aea = AllElectronAtom(Z, dirac=True, log=None)
aea.initialize(**kwargs)

errors = []
for channel in aea.channels:
    channel.solve(-Z)
    for n in range(7):
        e = channel.e_n[n]
        if channel.k > 0:
            n += 1
        e0 = (1 +
              (Z / c)**2 /
              ((channel.k**2 - (Z / c)**2)**0.5 + n)**2)**-0.5 - 1
        e0 *= c**2
        errors.append(abs(e / e0 - 1))
print(max(errors))
equal(max(errors), 0, 4.0e-5)



import numpy as np

from ase.build import molecule
from gpaw import GPAW
from gpaw.lcaotddft import LCAOTDDFT
from gpaw.poisson import PoissonSolver
from gpaw.lcaotddft.dipolemomentwriter import DipoleMomentWriter
from gpaw.mpi import world

from gpaw.test import equal

# Atoms
atoms = molecule('NaCl')
atoms.center(vacuum=4.0)

# Ground-state calculation
calc = GPAW(nbands=7, h=0.4, setups=dict(Na='1'),
            basis='dzp', mode='lcao',
            poissonsolver=PoissonSolver(eps=1e-16),
            convergence={'density': 1e-8},
            txt='gs.out')
atoms.set_calculator(calc)
energy = atoms.get_potential_energy()
calc.write('gs.gpw', mode='all')

# Reference time-propagation calculation
td_calc = LCAOTDDFT('gs.gpw', txt='td.out')
DipoleMomentWriter(td_calc, 'dm.dat')
td_calc.absorption_kick(np.ones(3) * 1e-5)
td_calc.propagate(20, 3)

# Check dipole moment file
world.barrier()
ref = np.loadtxt('dm.dat').ravel()

# Test parallelization options
par_i = []

if world.size == 2:
    par_i.append({'band': 2})
    par_i.append({'sl_default': (2, 1, 2)})
    par_i.append({'sl_default': (1, 2, 4), 'band': 2})
elif world.size == 4:
    par_i.append({'band': 2})
    par_i.append({'sl_default': (2, 2, 2)})
    par_i.append({'sl_default': (2, 2, 4), 'band': 2})
else:
    par_i.append({'band': 2})
    par_i.append({'sl_auto': True})
    par_i.append({'sl_auto': True, 'band': 2})

for i, par in enumerate(par_i):
    td_calc = LCAOTDDFT('gs.gpw', parallel=par, txt='td%d.out' % i)
    DipoleMomentWriter(td_calc, 'dm%d.dat' % i)
    td_calc.absorption_kick(np.ones(3) * 1e-5)
    td_calc.propagate(20, 3)

    world.barrier()
    data = np.loadtxt('dm%d.dat' % i).ravel()

    tol = 1e-11
    equal(data, ref, tol)



from ase.build import molecule
from ase.parallel import parprint

from gpaw import GPAW
from gpaw.cluster import Cluster
from gpaw.analyse.hirshfeld import HirshfeldDensity, HirshfeldPartitioning
from gpaw.analyse.wignerseitz import WignerSeitz
from gpaw.test import equal

h = 0.4
gpwname = 'H2O' + str(h) + '.gpw'

def run(lastres=[]):
    results = []

    # Hirshfeld ----------------------------------------

    if 1:

        hd = HirshfeldDensity(calc)

        # check for the number of electrons
        expected = [[None, 10],
                    [[0, 1, 2], 10],
                    [[1, 2], 2],
                    [[0], 8],
                    ]
        #expected = [[[0, 2], 9], ]
        #expected = [[None, 10], ]
        for gridrefinement in [1, 2, 4]:
            #Test for all gridrefinements for get_all_electron_density
            parprint('grid refinement', gridrefinement)
            for result in expected:
                indicees, result = result
                full, gd = hd.get_density(indicees, gridrefinement)
                parprint('indicees', indicees, end=': ') 
                parprint('result, expected:', gd.integrate(full), result)
                if gridrefinement < 4:
                    #The highest level of gridrefinement gets wrong electron numbers
                    equal(gd.integrate(full), result, 1.e-8)
                else:
                    equal(gd.integrate(full), result, 1.e-4)

        hp = HirshfeldPartitioning(calc)
        vr = hp.get_effective_volume_ratios()
        parprint('Hirshfeld:', vr)
        if len(lastres):
            equal(vr, lastres.pop(0), 1.e-10)
        results.append(vr)

    # Wigner-Seitz ----------------------------------------

    if 1:
        ws = WignerSeitz(calc.density.finegd, mol, calc)

        vr = ws.get_effective_volume_ratios()
        parprint('Wigner-Seitz:', vr)
        if len(lastres):
            equal(vr, lastres.pop(0), 1.e-10)
        results.append(vr)

    return results

mol = Cluster(molecule('H2O'))
mol.minimal_box(2.5, h=h)

# calculate
if 1:
    parprint('### fresh:')
    calc = GPAW(nbands=6,
                h = h, 
                txt=None)
if 1:
    calc.calculate(mol)
    calc.write(gpwname)
    lastres = run()

# load previous calculation
if 1:
    parprint('### reloaded:')
    calc = GPAW(gpwname, txt=None)
    mol = calc.get_atoms()
    run(lastres)

# periodic modulo test
parprint('### periodic:')
mol.set_pbc(True)
mol.translate(-mol[0].position)
mol.translate([-1.e-24, 0, 0])
calc.calculate(mol)
run()





from __future__ import print_function
from math import pi, cos, sin
from ase import Atom, Atoms
from gpaw import GPAW, PoissonSolver, Mixer, Davidson
from gpaw.test import equal, gen

# Generate setup for oxygen with half a core-hole:
gen('O', name='hch1s', corehole=(1, 0, 0.5))

a = 5.0
d = 0.9575
t = pi / 180 * 104.51
H2O = Atoms([Atom('O', (0, 0, 0)),
             Atom('H', (d, 0, 0)),
             Atom('H', (d * cos(t), d * sin(t), 0))],
            cell=(a, a, a), pbc=False)
H2O.center()
calc = GPAW(nbands=10, h=0.2, setups={'O': 'hch1s'},
            eigensolver=Davidson(4),
            mixer=Mixer(0.5),
            xc='oldLDA',
            poissonsolver=PoissonSolver(use_charge_center=True))
H2O.set_calculator(calc)
e = H2O.get_potential_energy()
niter = calc.get_number_of_iterations()
calc.write('h2o.gpw')

from gpaw.xas import RecursionMethod

if 1:
    r = RecursionMethod(calc)
    r.run(400)
    r.write('h2o.pckl')
else:
    r = RecursionMethod(filename='h2o.pckl')

print(e, niter)
energy_tolerance = 0.002
niter_tolerance = 0
equal(e, -17.980, energy_tolerance)



"""Trajectory module with viewmol support."""
from __future__ import print_function

from math import sqrt
import numpy as np

from ase import Angstrom, Atoms, Atom, Hartree, PickleTrajectory
from ase.io import filetype as ase_filetype
from ase.parallel import paropen
from ase.io.gpawtext import read_gpaw_text

from gpaw.cluster import Cluster
import gpaw.mpi as mpi
MASTER = 0

def filetype_from_ending(filename):
    # estimate file type from name ending
    ft = filename.split('.')[-1]
    return ft.lower()

class Trajectory(PickleTrajectory):
    """Basic trajectory class.

    In the moment, this class inherits from ase.PickleTrajectory.
    Might be other way round in some unknown future."""

    def __init__(self, filename=None, mode='r', atoms=None):
        self.images = []
        if filename is not None:
            self.read(filename)
        else:
            self.set_atoms(atoms)

    def __len__(self):
        if hasattr(self, 'offsets'):
            return PickleTrajectory.__len__(self)
        else:
            return len(self.images)

    def __del__(self):
        pass # overwrite PickleTrajectory.__del__

    def __getitem__(self, i=-1):
        if hasattr(self, 'offsets'):
            return PickleTrajectory.__getitem__(self, i)
        else:
            return self.images[i]

    def __iter__(self):
        if hasattr(self, 'offsets'):
            return PickleTrajectory.__iter__(self)
        else:
            return self.images.__iter__()

    def __next__(self):
        if hasattr(self, 'offsets'):
            return PickleTrajectory.next(self)
        else:
            return next(self.images)

    def append(self, images):
        if isinstance(images, type(Atoms)):
            images = [images]
        for image in images:
            self.images.append(image)

    def read(self, filename, filetype=None):
        """Read from a file"""

        ftfe = filetype_from_ending(filename)
        aft = ase_filetype(filename)

        if ftfe == 'vmol':
            self.read_viewmol(filename)
        elif aft == 'traj':
            PickleTrajectory.__init__(self, filename)
        elif aft == 'gpaw-text':
            # this is strange code, but I don't know how to create the full
            # slice XXXXX
            last = read_gpaw_text(filename)
            self.__init__(atoms=last)
            s = slice(-1)
            rest = read_gpaw_text(filename, s)
            rest.append(last)
            self.append(rest)
        else:
            raise NotImplementedError('unknown file type "'+ aft +'"')

    def write(self, filename, filetype=None):
        """Write to a file"""

        if filetype is None:
            filetype = filetype_from_ending(filename)
        if filetype == 'vmol':
            self.write_viewmol(filename)
        else:
            raise NotImplementedError

    def write_viewmol(self, filename, mode='w'):
        """Write yourself in viewmol style."""

        f = paropen(filename, mode)
        if mode == 'w':
            print(' $coord', 1. / Angstrom, file=f)
            self.write_viewmol_positions(f, self[0])
            print(' $grad', file=f)
        else:
            raise NotImplementedError('append')

        for i, atoms in enumerate(self):
            self.write_viewmol_atoms(f, atoms, i)

        f.close()

    def write_viewmol_atoms(self, file, atoms, index):
        """Write current atomic position, energy and forces
        to the output file"""
        print('cycle=', index, end=' ', file=file)
        try:
            E = atoms.get_potential_energy() * Hartree
        except:
            E = 0
        print('SCF energy=', E, end=' ', file=file)
        try:
            forces = atoms.get_forces() * (Hartree / Angstrom)
            max_force = 0
            for f in forces:
                max_force = max(max_force, sqrt(np.sum(f * f)))
        except:
            forces = atoms.get_positions() * 0.
            max_force = 0
        print('|max dE/dxyz|=', max_force, file=file)
        self.write_viewmol_positions(file, atoms)
        for atom, f in zip(atoms, forces):
            print('%10.4g %10.4g %10.4g' % (f[0],f[1],f[2]), file=file)

    def write_viewmol_positions(self, file, atoms):
        for c, s in zip(atoms.get_positions(), atoms.get_chemical_symbols()):
            print('%10.4f %10.4f %10.4f' % (c[0],c[1],c[2]), s, file=file)

    def read_viewmol(self, filename):
        f = open(filename)

        # read the definition first
        definition=False
        for line in f:
            w = line.split()
            if not definition:
                if w[0] == '$coord':
                    definition=True
                    self.scale = float(w[1])
                    loa = Cluster([])
            else:
                if w[0] == '$grad':
                    # definition ends here
                    self.definition = loa
                    break
                else:
                    # we assume this is a coordinate entry
                    coo = (float(w[0]),  float(w[1]), float(w[2]))
                    loa.append(Atom(w[3], coo))

        # get the iterations
        cycle = False
        for line in f:
            w = line.split()
            if not cycle:
                # search for the cycle keyword
                if w[0] == 'cycle=':
                    cycle=True
                    n_coo=0
                    n_F=0
                    self.images.append(Cluster([]))
            else:
                if n_coo < len(self.definition):
                    n_coo += 1
                    coo = (float(w[0]),  float(w[1]), float(w[2]))
                    self[-1].append(Atom(w[3], coo))
                elif n_F < len(self.definition):
                    F = (float(w[0]),  float(w[1]), float(w[2]))
                    self[-1][n_F].F = F
                    n_F += 1
                    if n_F == len(self.definition):
                        cycle=False

class ViewmolTrajectory(Trajectory):
    """Write a trajectory for viewmol (http://viewmol.sourceforge.net)

    You can attach the writing to the Calculator:

    from gpaw.utilities.viewmol import ViewmolTrajectory

    c = Calculator()
    H2 = Cluster([Atom('H',(0,0,-.9)),Atom('H',(0,0,.9))])
    H2.SetCalculator(c)

    vmt = ViewmolTrajectory(H2)
    c.attach(vmt.add,100000)
    """
    def __init__(self, atoms, filename='trajectory.vmol',mode='w'):
        self.Ha = 1.0 / Hartree
        self.Ang = Angstrom

        self.n = 0
        self.atoms = atoms
        if mpi.rank == MASTER:
            self.file = open(filename, mode)
        else:
            self.file = open('/dev/null', mode)
        print(' $coord', 1. / Angstrom, file=self.file)
        self.write_viewmol_positions(self.file, self.atoms)
        print(' $grad', file=self.file)

    def __del__(self):
        print(' $end', file=self.file)
        self.file.close()

    def add(self, atoms=None):
        """Write current atomic position to the output file"""
        if atoms is None:
            atoms = self.atoms
        self.n += 1
        print('cycle=', self.n, end=' ', file=self.file)
        print('SCF energy=', \
              atoms.get_potential_energy() * Hartree, end=' ', file=self.file)
        forces = atoms.get_forces() * (Hartree / Angstrom)
        max_force = 0
        for f in forces:
            max_force = max(max_force, sqrt(np.sum(f * f)))
        print('|max dE/dxyz|=', max_force, file=self.file)
        self.write_viewmol_positions(self.file, atoms)
        for atom, f in zip(self.atoms, forces):
            print('%10.4g %10.4g %10.4g' % (f[0],f[1],f[2]), file=self.file)
        self.file.flush()

    def read(self, filename='trajectory.vmol', position=0):
        """Read atom configurations of step position"""
        self.file = None
        f = open(filename)
        # find coordinates
        loa = Cluster([])
        coords = False
        for l in f.readlines():
            if coords:
                w = l.split()
                loa.append(Atom(w[3].replace("\n", ""),
                                (float(w[0]), float(w[1]), float(w[2]))))

def write_viewmol(pt, filename, mode='w'):
    """Write PickleTrajectory as viewmol file."""
    atoms = pt[0]
    vt = ViewmolTrajectory(atoms, filename, mode)
    for atoms in pt:
        vt.add(atoms)
    del(vt)




from __future__ import print_function
import numpy as np
from ase.build import bulk
from ase.dft.kpoints import monkhorst_pack
from ase.units import Bohr

from gpaw import GPAW, FermiDirac
from gpaw.response.chi0 import Chi0
from gpaw.mpi import serial_comm


omega = np.array([0, 1.0, 2.0])
for k in [2, 3]:
    q_c = [0, 0, 1.0 / k]
    for gamma in [False, True]:
        if k == 3 and gamma:
            continue
        kpts = monkhorst_pack((k, k, k))
        if gamma:
            kpts += 0.5 / k
        for center in [False, True]:
            a = bulk('Si', 'diamond')
            if center:
                a.center()
            for sym in [False, True]:
                name = 'si.k%d.g%d.c%d.s%d' % (k, gamma, center, bool(sym))
                print(name)
                if 1:
                    calc = a.calc = GPAW(
                        kpts=kpts,
                        eigensolver='rmm-diis',
                        symmetry={'point_group': sym},
                        mode='pw',
                        occupations=FermiDirac(width=0.001),
                        txt=name + '.txt')
                    e = a.get_potential_energy()
                    calc.write(name, 'all')
                    
                calc = GPAW(name, txt=None, communicator=serial_comm)

                chi = Chi0(calc, frequencies=omega, hilbert=False,
                           ecut=100, txt=name + '.log')
                pd, chi0_wGG, _, _ = chi.calculate(q_c)
                
                if not sym and not center:
                    chi00_w = chi0_wGG[:, 0, 0]
                elif -1 not in calc.wfs.kd.bz2bz_ks:
                    assert abs(chi0_wGG[:, 0, 0] - chi00_w).max() < 3e-5
                    
                if not sym:
                    chi00_wGG = chi0_wGG
                elif -1 not in calc.wfs.kd.bz2bz_ks:
                    assert abs(chi0_wGG - chi00_wGG).max() < 2e-5

                q0_c = [0, 1e-7, 1e-7]
                q0_v = np.dot(q0_c, a.get_reciprocal_cell() * 2 * np.pi) * Bohr
                q0 = (q0_v**2).sum()**0.5
                
                pd, chi0_wGG, _, _ = chi.calculate([0, 0, 0])
                
                if not sym and not center:
                    chi000_w = chi0_wGG[:, 0, 0]
                elif -1 not in calc.wfs.kd.bz2bz_ks:
                    assert abs(chi0_wGG[:, 0, 0] - chi000_w).max() < 0.001
                    
                if not sym:
                    chi000_wGG = chi0_wGG
                elif -1 not in calc.wfs.kd.bz2bz_ks:
                    assert abs(chi0_wGG - chi000_wGG).max() < 0.001



from ase.spacegroup import crystal
from gpaw import GPAW
from gpaw import PW
from gpaw.test import equal

name = 'cristobalite'
# no. 92 - tetragonal

a = 5.0833674
c = 7.0984738
p0 = (0.2939118, 0.2939118, 0.0)
p1 = (0.2412656, 0.0931314, 0.1739217)

atoms = crystal(['Si', 'O'], basis=[p0, p1],
                spacegroup=92, cellpar=[a, a, c, 90, 90, 90])


## with fractional translations
calc = GPAW(mode=PW(),
            xc='LDA',
            kpts=(3, 3, 2),
            nbands=40,
            symmetry={'symmorphic': False},
            gpts=(24, 24, 32),
            eigensolver='rmm-diis')

atoms.set_calculator(calc)
energy_fractrans = atoms.get_potential_energy()

assert(len(calc.wfs.kd.ibzk_kc) == 3)
assert(len(calc.wfs.kd.symmetry.op_scc) == 8)

## without fractional translations
calc = GPAW(mode=PW(),
            xc='LDA',
            kpts=(3, 3, 2),
            nbands=40,
            gpts=(24, 24, 32),
            eigensolver='rmm-diis')

atoms.set_calculator(calc)
energy_no_fractrans = atoms.get_potential_energy()

assert(len(calc.wfs.kd.ibzk_kc) == 6)
assert(len(calc.wfs.kd.symmetry.op_scc) == 2)

equal(energy_fractrans, energy_no_fractrans, 1e-7)



from __future__ import print_function
import numpy as np
from ase.build import fcc111

from gpaw.mpi import world
from gpaw import GPAW, PoissonSolver
from gpaw.utilities import compiled_with_sl

# This test verifies that energy and forces are (approximately)
# parallelization independent
#
# Tests the LCAO energy and forces in non-orthogonal cell with
# simultaneous parallelization over bands, domains and k-points (if
# enough CPUs are available), both with and without scalapack
# (if scalapack is available).
#
# Run with 1, 2, 4 or 8 (best) CPUs.
#
# This test covers many cases not caught by lcao_parallel or
# lcao_parallel_kpt
#
# Written November 24, 2011, r8567

system = fcc111('Au', size=(1, 3, 1))
system.numbers[0] = 8
# It is important that the number of atoms is uneven; this
# tests the case where the band parallelization does not match
# the partitioning of orbitals between atoms (the middle atom has orbitals
# on distinct band descriptor ranks)

system.center(vacuum=3.5, axis=2)
system.rattle(stdev=0.2, seed=17)
#from ase.visualize import view
#view(system)

#system.set_pbc(0)
#system.center(vacuum=3.5)
from gpaw import FermiDirac, LCAO

def calculate(parallel, comm=world, Eref=None, Fref=None):
    calc = GPAW(mode=LCAO(atomic_correction='sparse'),
                basis=dict(O='dzp', Au='sz(dzp)'),
                occupations=FermiDirac(0.1),
                kpts=(4, 1, 1),
                #txt=None,
                communicator=comm,
                poissonsolver=PoissonSolver(eps=1e-8),
                nbands=16,
                parallel=parallel,
                h=0.35)
    system.set_calculator(calc)
    E = system.get_potential_energy()
    F = system.get_forces()

    if world.rank == 0:
        print('Results')
        print('-----------')
        print(E)
        print(F)
        print('-----------')

    if Eref is not None:
        Eerr = abs(E - Eref)
        assert Eerr < 1e-8, 'Bad E: err=%f; parallel=%s' % (Eerr, parallel)
    if Fref is not None:
        Ferr = np.abs(F - Fref).max()
        assert Ferr < 1e-6, 'Bad F: err=%f; parallel=%s' % (Ferr, parallel)
    return E, F

# First calculate reference energy and forces E and F
#
# If we want to really dumb things down, enable this to force an
# entirely serial calculation:
if 0: 
    serial = world.new_communicator([0])
    E = 0.0
    F = np.zeros((len(system), 3))
    if world.rank == 0:
        E, F = calculate({}, serial)
    E = world.sum(E)
    world.sum(F)
else:
    # Normally we'll just do it in parallel;
    # that case is covered well by other tests, so we can probably trust it
    E, F = calculate({}, world)

def check(parallel):
    return calculate(parallel, comm=world, Eref=E, Fref=F)

assert world.size in [1, 2, 4, 8], ('Number of CPUs %d not supported'
                                    % world.size)

parallel = dict(domain=1, band=1)
sl_cpus = world.size
if world.size % 2 == 0:
    parallel['band'] = 2
if world.size % 4 == 0:
    parallel['domain'] = 2

# If size is 8, this will also use kpt parallelization.  This test should
# run with 8 CPUs for best coverage of parallelizations
if world.size == 8:
    sl_cpus = 4

if world.size > 1:
    check(parallel)

if compiled_with_sl():
    parallel['sl_auto'] = True
    check(parallel)



# Copyright (C) 2003  CAMP
# Please see the accompanying LICENSE file for further information.

"""This module defines a ``KPoint`` class and the derived ``GlobalKPoint``."""

import numpy as np


class KPoint:
    """Class for a single k-point.

    The KPoint class takes care of all wave functions for a
    certain k-point and a certain spin.

    XXX This needs to be updated.

    Attributes:

    phase_cd: complex ndarray
        Bloch phase-factors for translations - axis c=0,1,2
        and direction d=0,1.
    eps_n: float ndarray
        Eigenvalues.
    f_n: float ndarray
        Occupation numbers.  The occupation numbers already include the
        k-point weight in supercell calculations.
    psit_nG: ndarray
        Wave functions.
    nbands: int
        Number of bands.

    Parallel stuff:

    comm: Communicator object
        MPI-communicator for domain.
    root: int
        Rank of the CPU that does the matrix diagonalization of
        H_nn and the Cholesky decomposition of S_nn.
    """

    def __init__(self, weight, s, k, q, phase_cd):
        """Construct k-point object.

        Parameters:

        gd: GridDescriptor object
            Descriptor for wave-function grid.
        weight: float
            Weight of this k-point.
        s: int
            Spin index: up or down (0 or 1).
        k: int
            k-point index.
        q: int
            local k-point index.
        dtype: type object
            Data type of wave functions (float or complex).
        timer: Timer object
            Optional.

        Note that s and k are global spin/k-point indices,
        whereas u is a local spin/k-point pair index for this
        processor.  So if we have `S` spins and `K` k-points, and
        the spins/k-points are parallelized over `P` processors
        (kpt_comm), then we have this equation relating s,
        k and u::

           rSK
           --- + u = sK + k,
            P

        where `r` is the processor rank within kpt_comm.  The
        total number of spin/k-point pairs, `SK`, is always a
        multiple of the number of processors, `P`.
        """

        self.weight = weight
        self.s = s  # spin index
        self.k = k  # k-point index
        self.q = q  # local k-point index
        self.phase_cd = phase_cd

        self.eps_n = None
        self.f_n = None
        self.projections = None  # Projections

        # Only one of these two will be used:
        self.psit = None  # UniformGridMatrix/PWExpansionMatrix
        self.C_nM = None  # LCAO coefficients for wave functions

        # LCAO stuff:
        self.rho_MM = None
        self.S_MM = None
        self.T_MM = None

    @property
    def P_ani(self):
        if self.projections is not None:
            return {a: P_ni for a, P_ni in self.projections.items()}

    @property
    def psit_nG(self):
        if self.psit is not None:
            return self.psit.array


class GlobalKPoint(KPoint):
    def update(self, wfs):
        """Distribute requested kpoint data across the kpoint communicator."""

        # Locate rank and index of requested k-point
        nks = len(wfs.kd.ibzk_kc)
        mynu = len(wfs.kpt_u)
        kpt_rank, u = divmod(self.k + nks * self.s, mynu)
        kpt_comm = wfs.kd.comm

        my_atom_indices = wfs.atom_partition.my_atom_indices
        mynproj = sum([wfs.setups[a].ni for a in my_atom_indices])
        my_P_ni = np.empty((wfs.mynbands, mynproj), wfs.dtype)

        self.P_ani = {}

        if self.phase_cd is None:
            self.phase_cd = np.empty((3, 2), wfs.dtype)

        if self.psit_nG is None:
            self.psit_nG = wfs.gd.empty(wfs.mynbands, wfs.dtype)

        reqs = []

        # Do I have the requested kpoint?
        if kpt_comm.rank == kpt_rank:
            self.phase_cd[:] = wfs.kpt_u[u].phase_cd
            self.psit_nG[:] = wfs.kpt_u[u].psit_nG

            # Compress entries in requested P_ani dict into my_P_ni ndarray
            i = 0
            for a, P_ni in wfs.kpt_u[u].P_ani.items():
                ni = wfs.setups[a].ni
                my_P_ni[:, i:i + ni] = P_ni
                i += ni

            assert (my_atom_indices == wfs.kpt_u[u].P_ani.keys()).all()

            # Send phase_cd, psit_nG and my_P_ni to kpoint slaves
            for rank in range(kpt_comm.size):
                if rank != kpt_rank:
                    reqs.append(kpt_comm.send(self.phase_cd, rank, 256, False))
                    reqs.append(kpt_comm.send(self.psit_nG, rank, 257, False))
                    reqs.append(kpt_comm.send(my_P_ni, rank, 258, False))
        else:
            # Receive phase_cd, psit_nG and my_P_ni from kpoint master
            reqs.append(kpt_comm.receive(self.phase_cd, kpt_rank, 256, False))
            reqs.append(kpt_comm.receive(self.psit_nG, kpt_rank, 257, False))
            reqs.append(kpt_comm.receive(my_P_ni, kpt_rank, 258, False))

        for request in reqs:
            kpt_comm.wait(request)

        # Decompress my_P_ni ndarray into entries in my P_ani dict
        i = 0
        for a in my_atom_indices:
            ni = wfs.setups[a].ni
            self.P_ani[a] = my_P_ni[:, i:i + ni]  # copy?
            i += ni



from __future__ import print_function
import numpy as np
from gpaw.test import equal
from gpaw.grid_descriptor import GridDescriptor
from gpaw.wavefunctions.pw import PWDescriptor
from gpaw.mpi import world


def test(gd1, gd2, pd1, pd2, R1, R2):
    a1 = gd1.zeros(dtype=pd1.dtype)
    a1[R1] = 1
    a2 = pd1.interpolate(a1, pd2)[0]
    x = a2[R2]
    
    a2 = gd2.zeros(dtype=pd2.dtype)
    a2[R2] = 1
    y = pd2.restrict(a2, pd1)[0][R1] * a2.size / a1.size

    equal(x, y, 1e-9)
    return x

if world.size == 1:
    for size1, size2 in [
        [(3, 3, 3), (8, 8, 8)],
        [(4, 4, 4), (9, 9, 9)],
        [(2, 4, 4), (5, 9, 9)],
        [(2, 3, 4), (5, 6, 9)],
        [(2, 3, 4), (5, 6, 8)],
        [(4, 4, 4), (8, 8, 8)],
        [(2, 4, 4), (4, 8, 8)],
        [(2, 4, 2), (4, 8, 4)]]:
        print(size1, size2)
        gd1 = GridDescriptor(size1, size1)
        gd2 = GridDescriptor(size2, size1)
        pd1 = PWDescriptor(1, gd1, complex)
        pd2 = PWDescriptor(1, gd2, complex)
        pd1r = PWDescriptor(1, gd1)
        pd2r = PWDescriptor(1, gd2)
        for R1, R2 in [[(0, 0, 0), (0, 0, 0)],
                       [(0, 0, 0), (0, 0, 1)]]:
            x = test(gd1, gd2, pd1, pd2, R1, R2)
            y = test(gd1, gd2, pd1r, pd2r, R1, R2)
            equal(x, y, 1e-9)
        
        a1 = np.random.random(size1)
        a2 = pd1r.interpolate(a1, pd2r)[0]
        c2 = pd1.interpolate(a1 + 0.0j, pd2)[0]
        d2 = pd1.interpolate(a1 * 1.0j, pd2)[0]
        equal(abs(c2.imag).max(), 0, 1e-14)
        equal(abs(d2.real).max(), 0, 1e-14)
        equal(gd1.integrate(a1), gd2.integrate(a2), 1e-13)
        equal(abs(c2 - a2).max(), 0, 1e-14)
        equal(abs(d2 - a2 * 1.0j).max(), 0, 1e-14)

        a1 = pd2r.restrict(a2, pd1r)[0]
        c1 = pd2.restrict(a2 + 0.0j, pd1)[0]
        d1 = pd2.restrict(a2 * 1.0j, pd1)[0]
        equal(gd1.integrate(a1), gd2.integrate(a2), 1e-13)
        equal(abs(c1 - a1).max(), 0, 1e-14)
        equal(abs(d1 - a1 * 1.0j).max(), 0, 1e-14)



"""Test EXX/HFT implementation."""
from __future__ import print_function
from ase import Atoms
from gpaw import GPAW
from gpaw.test import equal

be2 = Atoms('Be2', [(0, 0, 0), (2.45, 0, 0)])
be2.center(vacuum=2.0)
calc = GPAW(h=0.21,
            eigensolver='rmm-diis',
            nbands=3,
            convergence={'eigenstates': 1e-6},
            txt='exx.txt')

be2.set_calculator(calc)

ref_1871 = {  # Values from revision 1871. Not true reference values
    # xc         Energy          eigenvalue 0    eigenvalue 1
    'PBE': (5.424066548470926, -3.84092, -0.96192),
    'PBE0': (-790.919942, -4.92321, -1.62948),
    'EXX': (-785.5837828306236, -7.16802337336, -2.72602997017)
    }

def xc(name):
    return dict(name=name, stencil=1)

from gpaw.xc import XC
from gpaw.xc.hybrid import HybridXC
current = {}  # Current revision
for xc in [XC(xc('PBE')),
           HybridXC('PBE0', stencil=1, finegrid=True),
           HybridXC('EXX', stencil=1, finegrid=True),
           XC(xc('PBE'))]:  # , 'oldPBE', 'LDA']:
    # Generate setup
    #g = Generator('Be', setup, scalarrel=True, nofiles=True, txt=None)
    #g.run(exx=True, **parameters['Be'])

    # switch to new xc functional
    calc.set(xc=xc)
    E = be2.get_potential_energy()
    if xc.name != 'PBE':
        E += calc.get_reference_energy()
    bands = calc.get_eigenvalues()[:2]  # not 3 as unocc. eig are random!? XXX
    res = (E,) + tuple(bands)
    print(xc.name, res)

    if xc.name in current:
        for first, second in zip(current[xc.name], res):
            equal(first, second, 2.5e-3)
    else:
        current[xc.name] = res

for name in current:
    for ref, cur in zip(ref_1871[name], current[name]):
        print(ref, cur, ref - cur)
        equal(ref, cur, 2.9e-3)



from ase import Atoms
from gpaw import GPAW
from gpaw.test import equal

for mode in ['fd', 'pw']:
    print(mode)
    hydrogen = Atoms('H',
                     cell=(2.5, 3, 3.5),
                     pbc=1,
                     calculator=GPAW(txt=None, mode=mode))
    hydrogen.get_potential_energy()
    dens = hydrogen.calc.density
    ham = hydrogen.calc.hamiltonian
    ham.poisson.eps = 1e-20
    dens.interpolate_pseudo_density()
    dens.calculate_pseudo_charge()
    ham.update(dens)
    ham.get_energy(hydrogen.calc.occupations)
    y = (ham.vt_sG[0, 0, 0, 0] - ham.vt_sG[0, 0, 0, 1]) * ham.gd.dv
    x = 0.0001
    dens.nt_sG[0, 0, 0, 0] += x
    dens.nt_sG[0, 0, 0, 1] -= x
    dens.interpolate_pseudo_density()
    dens.calculate_pseudo_charge()
    ham.update(dens)
    e1 = ham.get_energy(hydrogen.calc.occupations) - ham.e_kinetic
    dens.nt_sG[0, 0, 0, 0] -= 2 * x
    dens.nt_sG[0, 0, 0, 1] += 2 * x
    dens.interpolate_pseudo_density()
    dens.calculate_pseudo_charge()
    ham.update(dens)
    e2 = ham.get_energy(hydrogen.calc.occupations) - ham.e_kinetic
    equal(y, (e1 - e2) / (2 * x), 2e-8)



from ase import Atoms
from gpaw import GPAW
from gpaw.test import equal


# Self-consistent calculation:
a = 2.5
slab = Atoms('Li', cell=(a, a, 2 * a), pbc=1)
slab.calc = GPAW(kpts=(3, 3, 1), txt='li.txt',
                 parallel=dict(kpt=1))
slab.get_potential_energy()
slab.calc.write('li.gpw')

# Gamma point:
e1 = slab.calc.get_eigenvalues(kpt=0)[0]
f1 = slab.calc.get_fermi_level()

# Fix density and continue:
kpts = [(0, 0, 0)]
slab.calc.set(fixdensity=True,
              nbands=5,
              kpts=kpts,
              symmetry='off',
              eigensolver='cg')
slab.get_potential_energy()
e2 = slab.calc.get_eigenvalues(kpt=0)[0]
f2 = slab.calc.get_fermi_level()

# Start from gpw-file:
calc = GPAW('li.gpw',
            txt='li2.txt',
            fixdensity=True,
            nbands=5,
            kpts=kpts,
            symmetry='off',
            eigensolver='cg')

calc.get_potential_energy()
e3 = calc.get_eigenvalues(kpt=0)[0]
f3 = slab.calc.get_fermi_level()

equal(f2, f1, 1e-10)
equal(f3, f1, 1e-10)
equal(e1, e2, 3e-5)
equal(e1, e3, 3e-5)



from ase.build import bulk
from gpaw import GPAW
from gpaw.mixer import Mixer
from gpaw.test import equal
from gpaw.test import gen


symbol = 'C'
result = -224.206
electrons = 48

xcname = 'LDA_K_TF+LDA_X'
g = gen(symbol, xcname=xcname, scalarrel=False, orbital_free=True)
h = 0.14
a = 2.8
atoms = bulk(symbol, 'diamond', a=a, cubic=True)   # Generate diamond
mixer = Mixer(0.1, 5)

calc = GPAW(h=h,
            xc=xcname,
            maxiter=120,
            eigensolver='cg',
            mixer=mixer)


atoms.set_calculator(calc)

e = atoms.get_potential_energy()

n = calc.get_all_electron_density()

dv = atoms.get_volume() / calc.get_number_of_grid_points().prod()
I = n.sum() * dv / 2**3

equal(I, electrons, 1.0e-6)
equal(e, result, 1.0e-2)
